# -*- coding: utf-8 -*-

from __future__ import (absolute_import, division, print_function)
from ._version import LooseVersion
__metaclass__ = type


class CommandResolver():
    def __init__(self, server_implementation, server_version):
        self.server_implementation = server_implementation
        self.server_version = LooseVersion(server_version)

    def resolve_command(self, command):
        """
        Resolves the appropriate SQL command based on the server implementation and version.

        Parameters:
        command (str): The base SQL command to be resolved (e.g., "SHOW SLAVE HOSTS").

        Returns:
        str: The resolved SQL command suitable for the given server implementation and version.

        Raises:
        ValueError: If the command is not supported or recognized.

        Example:
        Given a server implementation `mysql` and server version `8.0.23`, and a command `SHOW SLAVE HOSTS`,
        the method will resolve the command based on the following table of versions:

        Table:
            [
                ("mysql", "default", "SHOW SLAVES HOSTS default"),
                ("mysql", "5.7.0", "SHOW SLAVES HOSTS"),
                ("mysql", "8.0.22", "SHOW REPLICAS"),
                ("mysql", "8.4.0", "SHOW REPLICAS  8.4"),
                ("mariadb", "10.5.1", "SHOW REPLICAS HOSTS"),
            ]

        Example usage:
        >>> resolver = CommandResolver("mysql", "8.0.23")
        >>> resolver.resolve_command("SHOW SLAVE HOSTS")
        'SHOW REPLICAS'

        In this example, the resolver will:
        - Filter and sort applicable versions: [
            ("8.4.0", "SHOW REPLICAS  8.4"),
            ("8.0.22", "HOW REPLICAS"),
            ("5.7.0", "SHOW SLAVES HOSTS")
          ]

        - Iterate through the sorted list and find the first version less than or equal to 8.0.23,
          which is 8.0.22, and return the corresponding command.
        """

        # Convert the command to uppercase to ensure case-insensitive lookup
        command = command.upper()

        commands = {
            "SHOW MASTER STATUS": {
                ("mysql", "default"): "SHOW MASTER STATUS",
                ("mariadb", "default"): "SHOW MASTER STATUS",
                ("mysql", "8.2.0"): "SHOW BINARY LOG STATUS",
                ("mariadb", "10.5.2"): "SHOW BINLOG STATUS",
            },
            "SHOW SLAVE STATUS": {
                ("mysql", "default"): "SHOW SLAVE STATUS",
                ("mariadb", "default"): "SHOW SLAVE STATUS",
                ("mysql", "8.0.22"): "SHOW REPLICA STATUS",
                ("mariadb", "10.5.1"): "SHOW REPLICA STATUS",
            },
            "SHOW SLAVE HOSTS": {
                ("mysql", "default"): "SHOW SLAVE HOSTS",
                ("mariadb", "default"): "SHOW SLAVE HOSTS",
                ("mysql", "8.0.22"): "SHOW REPLICAS",
                ("mariadb", "10.5.1"): "SHOW REPLICA HOSTS",
            },
            "CHANGE MASTER": {
                ("mysql", "default"): "CHANGE MASTER",
                ("mariadb", "default"): "CHANGE MASTER",
                ("mysql", "8.0.23"): "CHANGE REPLICATION SOURCE",
            },
            "MASTER_HOST": {
                ("mysql", "default"): "MASTER_HOST",
                ("mariadb", "default"): "MASTER_HOST",
                ("mysql", "8.0.23"): "SOURCE_HOST",
            },
            "MASTER_USER": {
                ("mysql", "default"): "MASTER_USER",
                ("mariadb", "default"): "MASTER_USER",
                ("mysql", "8.0.23"): "SOURCE_USER",
            },
            "MASTER_PASSWORD": {
                ("mysql", "default"): "MASTER_PASSWORD",
                ("mariadb", "default"): "MASTER_PASSWORD",
                ("mysql", "8.0.23"): "SOURCE_PASSWORD",
            },
            "MASTER_PORT": {
                ("mysql", "default"): "MASTER_PORT",
                ("mariadb", "default"): "MASTER_PORT",
                ("mysql", "8.0.23"): "SOURCE_PORT",
            },
            "MASTER_CONNECT_RETRY": {
                ("mysql", "default"): "MASTER_CONNECT_RETRY",
                ("mariadb", "default"): "MASTER_CONNECT_RETRY",
                ("mysql", "8.0.23"): "SOURCE_CONNECT_RETRY",
            },
            "MASTER_LOG_FILE": {
                ("mysql", "default"): "MASTER_LOG_FILE",
                ("mariadb", "default"): "MASTER_LOG_FILE",
                ("mysql", "8.0.23"): "SOURCE_LOG_FILE",
            },
            "MASTER_LOG_POS": {
                ("mysql", "default"): "MASTER_LOG_POS",
                ("mariadb", "default"): "MASTER_LOG_POS",
                ("mysql", "8.0.23"): "SOURCE_LOG_POS",
            },
            "MASTER_DELAY": {
                ("mysql", "default"): "MASTER_DELAY",
                ("mariadb", "default"): "MASTER_DELAY",
                ("mysql", "8.0.23"): "SOURCE_DELAY",
            },
            "MASTER_SSL": {
                ("mysql", "default"): "MASTER_SSL",
                ("mariadb", "default"): "MASTER_SSL",
                ("mysql", "8.0.23"): "SOURCE_SSL",
            },
            "MASTER_SSL_CA": {
                ("mysql", "default"): "MASTER_SSL_CA",
                ("mariadb", "default"): "MASTER_SSL_CA",
                ("mysql", "8.0.23"): "SOURCE_SSL_CA",
            },
            "MASTER_SSL_CAPATH": {
                ("mysql", "default"): "MASTER_SSL_CAPATH",
                ("mariadb", "default"): "MASTER_SSL_CAPATH",
                ("mysql", "8.0.23"): "SOURCE_SSL_CAPATH",
            },
            "MASTER_SSL_CERT": {
                ("mysql", "default"): "MASTER_SSL_CERT",
                ("mariadb", "default"): "MASTER_SSL_CERT",
                ("mysql", "8.0.23"): "SOURCE_SSL_CERT",
            },
            "MASTER_SSL_KEY": {
                ("mysql", "default"): "MASTER_SSL_KEY",
                ("mariadb", "default"): "MASTER_SSL_KEY",
                ("mysql", "8.0.23"): "SOURCE_SSL_KEY",
            },
            "MASTER_SSL_CIPHER": {
                ("mysql", "default"): "MASTER_SSL_CIPHER",
                ("mariadb", "default"): "MASTER_SSL_CIPHER",
                ("mysql", "8.0.23"): "SOURCE_SSL_CIPHER",
            },
            "MASTER_SSL_VERIFY_SERVER_CERT": {
                ("mysql", "default"): "MASTER_SSL_VERIFY_SERVER_CERT",
                ("mariadb", "default"): "MASTER_SSL_VERIFY_SERVER_CERT",
                ("mysql", "8.0.23"): "SOURCE_SSL_VERIFY_SERVER_CERT",
            },
            "MASTER_AUTO_POSITION": {
                ("mysql", "default"): "MASTER_AUTO_POSITION",
                ("mariadb", "default"): "MASTER_AUTO_POSITION",
                ("mysql", "8.0.23"): "SOURCE_AUTO_POSITION",
            },
            "RESET MASTER": {
                ("mysql", "default"): "RESET MASTER",
                ("mariadb", "default"): "RESET MASTER",
                ("mysql", "8.4.0"): "RESET BINARY LOGS AND GTIDS",
            },
            # Add more command mappings here
        }

        if command in commands:
            cmd_syntaxes = commands[command]
            applicable_versions = [(v, cmd) for (impl, v), cmd in cmd_syntaxes.items() if impl == self.server_implementation and v != 'default']
            applicable_versions.sort(reverse=True, key=lambda x: LooseVersion(x[0]))

            for version, cmd in applicable_versions:
                if self.server_version >= LooseVersion(version):
                    return cmd

            return cmd_syntaxes[(self.server_implementation, "default")]
        raise ValueError("Unsupported command: %s" % command)



# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.ansible.mysql.plugins.module_utils.version import LooseVersion
from ansible_collections.ansible.mysql.plugins.module_utils.mysql import get_server_version

import re
import shlex


def use_old_user_mgmt(cursor):
    version = get_server_version(cursor)

    return LooseVersion(version) < LooseVersion("5.7")


def supports_identified_by_password(cursor):
    version = get_server_version(cursor)
    return LooseVersion(version) < LooseVersion("8")


def server_supports_alter_user(cursor):
    version = get_server_version(cursor)

    return LooseVersion(version) >= LooseVersion("5.6")


def server_supports_password_expire(cursor):
    version = get_server_version(cursor)

    return LooseVersion(version) >= LooseVersion("5.7")


def get_tls_requires(cursor, user, host):
    """Get user TLS requirements.
    We must use SHOW GRANTS because some tls fileds are encoded.

    Args:
        cursor (cursor): DB driver cursor object.
        user (str): User name.
        host (str): User host name.

    Returns: Dictionary containing current TLS required
    """
    if not use_old_user_mgmt(cursor):
        query = "SHOW CREATE USER '%s'@'%s'" % (user, host)
    else:
        query = "SHOW GRANTS for '%s'@'%s'" % (user, host)

    cursor.execute(query)
    grants = cursor.fetchone()

    # Mysql_info use a DictCursor so we must convert back to a list
    # otherwise we get KeyError 0
    if isinstance(grants, dict):
        grants = list(grants.values())
    grants_str = ''.join(grants)

    pattern = r"(?<=\bREQUIRE\b)(.*?)(?=(?:\bPASSWORD\b|$))"
    requires_match = re.search(pattern, grants_str)
    requires = requires_match.group().strip() if requires_match else ""

    if requires.startswith('NONE'):
        return None

    if requires.startswith('SSL'):
        return {'SSL': None}

    if requires.startswith('X509'):
        return {'X509': None}

    items = iter(shlex.split(requires))
    requires = dict(zip(items, items))
    return requires or None



#!/usr/bin/python

# Copyright (c) 2017, Eike Frost <ei@kefro.st>
# Copyright (c) 2021, Christophe Gilles <christophe.gilles54@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or
# https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: keycloak_authz_permission_info

version_added: 7.2.0

short_description: Query Keycloak client authorization permissions information

description:
  - This module allows querying information about Keycloak client authorization permissions from the resources endpoint using
    the Keycloak REST API. Authorization permissions are only available if a client has Authorization enabled.
  - This module requires access to the REST API using OpenID Connect; the user connecting and the realm being used must have
    the requisite access rights. In a default Keycloak installation, admin-cli and an admin user would work, as would a separate
    realm definition with the scope tailored to your needs and a user having the expected roles.
  - The names of module options are snake_cased versions of the camelCase options used by Keycloak. The Authorization Services
    paths and payloads have not officially been documented by the Keycloak project.
    U(https://www.puppeteers.net/blog/keycloak-authorization-services-rest-api-paths-and-payload/).
attributes:
  action_group:
    version_added: 10.2.0

options:
  name:
    description:
      - Name of the authorization permission to create.
    type: str
    required: true
  client_id:
    description:
      - The clientId of the keycloak client that should have the authorization scope.
      - This is usually a human-readable name of the Keycloak client.
    type: str
    required: true
  realm:
    description:
      - The name of the Keycloak realm the Keycloak client is in.
    type: str
    required: true

extends_documentation_fragment:
  - community.general._keycloak
  - community.general._keycloak.actiongroup_keycloak
  - community.general._attributes
  - community.general._attributes.info_module

author:
  - Samuli Seppänen (@mattock)
"""

EXAMPLES = r"""
- name: Query Keycloak authorization permission
  community.general.keycloak_authz_permission_info:
    name: ScopePermission
    client_id: myclient
    realm: myrealm
    auth_keycloak_url: http://localhost:8080/auth
    auth_username: keycloak
    auth_password: keycloak
    auth_realm: master
"""

RETURN = r"""
msg:
  description: Message as to what action was taken.
  returned: always
  type: str

queried_state:
  description: State of the resource (a policy) as seen by Keycloak.
  returned: on success
  type: dict
  contains:
    id:
      description: ID of the authorization permission.
      type: str
      sample: 9da05cd2-b273-4354-bbd8-0c133918a454
    name:
      description: Name of the authorization permission.
      type: str
      sample: ResourcePermission
    description:
      description: Description of the authorization permission.
      type: str
      sample: Resource Permission
    type:
      description: Type of the authorization permission.
      type: str
      sample: resource
    decisionStrategy:
      description: The decision strategy.
      type: str
      sample: UNANIMOUS
    logic:
      description: The logic used for the permission (part of the payload, but has a fixed value).
      type: str
      sample: POSITIVE
    config:
      description: Configuration of the permission (empty in all observed cases).
      type: dict
      sample: {}
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._keycloak import (
    KeycloakAPI,
    KeycloakError,
    get_token,
    keycloak_argument_spec,
)


def main():
    """
    Module execution

    :return:
    """
    argument_spec = keycloak_argument_spec()

    meta_args = dict(
        name=dict(type="str", required=True),
        client_id=dict(type="str", required=True),
        realm=dict(type="str", required=True),
    )

    argument_spec.update(meta_args)

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=(
            [["token", "auth_realm", "auth_username", "auth_password", "auth_client_id", "auth_client_secret"]]
        ),
        required_together=([["auth_username", "auth_password"]]),
        required_by={"refresh_token": "auth_realm"},
    )

    # Convenience variables
    name = module.params.get("name")
    client_id = module.params.get("client_id")
    realm = module.params.get("realm")

    result = dict(changed=False, msg="", queried_state={})

    # Obtain access token, initialize API
    try:
        connection_header = get_token(module.params)
    except KeycloakError as e:
        module.fail_json(msg=str(e))

    kc = KeycloakAPI(module, connection_header)

    # Get id of the client based on client_id
    cid = kc.get_client_id(client_id, realm=realm)
    if not cid:
        module.fail_json(msg=f"Invalid client {client_id} for realm {realm}")

    # Get current state of the permission using its name as the search
    # filter. This returns False if it is not found.
    permission = kc.get_authz_permission_by_name(name=name, client_id=cid, realm=realm)

    result["queried_state"] = permission

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: backup_restore_job_info
version_added: 6.0.0
short_description: List information about backup restore jobs
description:
    - List detailed information about AWS Backup restore jobs initiated to restore a saved resource.
author:
    - Mandar Vijay Kulkarni (@mandar242)
options:
  account_id:
    description:
      - The account ID to list the restore jobs from.
    required: false
    type: str
  status:
    description:
      - Status of restore jobs to filter the result based on job status.
    required: false
    choices: ['PENDING', 'RUNNING', 'COMPLETED', 'ABORTED', 'FAILED']
    type: str
  created_before:
    description:
      - Specified date to filter result based on the restore job creation datetime.
      - If specified, only the restore jobs created before the specified datetime will be returned.
      - The date must be in Unix format and Coordinated Universal Time (UTC), example "2023-02-25T00:05:36.309Z".
    required: false
    type: str
  created_after:
    description:
      - Specified date to filter result based on the restore job creation datetime.
      - If specified, only the restore jobs created after the specified datetime will be returned.
      - The date must be in Unix format and Coordinated Universal Time (UTC), example "2023-02-25T00:05:36.309Z".
    required: false
    type: str
  completed_before:
    description:
      - Specified date to filter result based on the restore job completion datetime.
      - If specified, only the restore jobs created before the specified datetime will be returned.
      - The date must be in Unix format and Coordinated Universal Time (UTC), example "2023-02-25T00:05:36.309Z".
    required: false
    type: str
  completed_after:
    description:
      - Specified date to filter result based on the restore job completion datetime.
      - If specified, only the restore jobs created after the specified datetime will be returned.
      - The date must be in Unix format and Coordinated Universal Time (UTC), example "2023-02-25T00:05:36.309Z".
    required: false
    type: str
  restore_job_id:
      description:
        - ID of the restore job to get information about.
        - This parameter is mutually exlusive with all other parameters.
      required: false
      type: str
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: List all restore jobs
  amazon.aws.backup_restore_job_info:

- name: List specific restore job's info by job ID
  amazon.aws.backup_restore_job_info:
    restore_job_id: "52BEE289-xxxx-xxxx-xxxx-47DCAA2E7ACD"

- name: List restore jobs based on Account ID
  amazon.aws.backup_restore_job_info:
    account_id: xx1234567890

- name: List restore jobs based on status and created_before time
  amazon.aws.backup_restore_job_info:
    status: completed
    created_before: "2023-02-25T00:05:36.309Z"
"""

RETURN = r"""
restore_jobs:
    returned: always
    description:
        - restore jobs that match the provided filters.
        - Each element consists of a dict with details related to that restore job.
    type: list
    elements: dict
    contains:
        account_id:
            description:
                - The account ID that owns the restore job.
            type: str
            returned: if restore job exists
            sample: "123456789012"
        created_resource_arn:
            description:
                - An Amazon Resource Name (ARN) that uniquely identifies a resource whose recovery point is being restored.
                - The format of the ARN depends on the resource type of the backed-up resource.
            type: str
            returned: if restore job exists
            sample: "arn:aws:ec2:us-east-2:xxxxxxxxxx..."
        creation_date:
            description:
                - The date and time that a restore job is created, in Unix format and Coordinated Universal Time (UTC).
            type: str
            returned: if restore job exists
            sample: "2023-03-13T15:53:07.172000-07:00"
        iam_role_arn:
            description:
                - The IAM role ARN used to create the target recovery point.
            type: str
            returned: if restore job exists
            sample: "arn:aws:ec2:us-east-2:xxxxxxxxxx..."
        percent_done:
            description:
                - The estimated percentage that is complete of a job at the time the job status was queried.
            type: str
            returned: if restore job exists
            sample: "0.00%"
        recovery_point_arn:
            description:
                - An ARN that uniquely identifies a recovery point.
            type: str
            returned: if restore job exists
            sample: "arn:aws:ec2:us-east-2:xxxxxxxxxx..."
        restore_job_id:
            description:
                - The ID of the job that restores a recovery point.
            type: str
            returned: if restore job exists
            sample: "AAAA1234-1D1D-1234-3F8E-1EB111EEEE00"
        status:
            description:
                - The state of the job initiated by Backup to restore a recovery point.
            type: str
            returned: if restore job exists
            sample: "COMPLETED"
"""

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


def build_request_args(account_id, status, created_before, created_after, completed_before, completed_after):
    request_args = {
        "ByAccountId": account_id if account_id else "",
        "ByStatus": status if status else "",
        "ByCreatedBefore": created_before if created_before else "",
        "ByCreatedAfter": created_after if created_after else "",
        "ByCompleteBefore": completed_before if completed_before else "",
        "ByCompleteAfter": completed_after if completed_after else "",
    }

    request_args = {k: v for k, v in request_args.items() if v}

    return request_args


def _describe_restore_job(connection, module, restore_job_id):
    try:
        response = connection.describe_restore_job(RestoreJobId=restore_job_id)
        response.pop("ResponseMetadata", None)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg=f"Failed to describe restore job with ID: {restore_job_id}")

    return [camel_dict_to_snake_dict(response)]


@AWSRetry.jittered_backoff()
def _list_restore_jobs(connection, **params):
    paginator = connection.get_paginator("list_restore_jobs")
    return paginator.paginate(**params).build_full_result()


def list_restore_jobs(connection, module, request_args):
    try:
        response = _list_restore_jobs(connection, **request_args)
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to list restore jobs")

    return [camel_dict_to_snake_dict(restore_job) for restore_job in response["RestoreJobs"]]


def main():
    argument_spec = dict(
        account_id=dict(required=False, type="str"),
        status=dict(required=False, type="str", choices=["PENDING", "RUNNING", "COMPLETED", "ABORTED", "FAILED"]),
        created_before=dict(required=False, type="str"),
        created_after=dict(required=False, type="str"),
        completed_before=dict(required=False, type="str"),
        completed_after=dict(required=False, type="str"),
        restore_job_id=dict(required=False, type="str"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    backup_client = module.client("backup")

    request_args = build_request_args(
        account_id=module.params["account_id"],
        status=module.params["status"],
        created_before=module.params["created_before"],
        created_after=module.params["created_after"],
        completed_before=module.params["completed_before"],
        completed_after=module.params["completed_after"],
    )

    if module.params.get("restore_job_id"):
        restore_jobs = _describe_restore_job(backup_client, module, module.params.get("restore_job_id"))
    else:
        restore_jobs = list_restore_jobs(backup_client, module, request_args)

    module.exit_json(changed=False, restore_jobs=restore_jobs)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_storage_policy
short_description: Storage Policy configuration for Cisco Intersight
description:
  - Manages Storage Policy configuration on Cisco Intersight.
  - A policy to configure storage settings and virtual drive configurations for Cisco Intersight managed servers.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Storage Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Storage Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
    default: []
  use_jbod_for_vd_creation:
    description:
      - Disks in JBOD State are used to create virtual drives.
      - This setting must be disabled if Default Drive State is set to JBOD.
    type: bool
    default: false
  unused_disks_state:
    description:
      - State to which drives, not used in this policy, are to be moved.
      - NoChange will not change the drive state.
      - No Change must be selected if Default Drive State is set to JBOD or RAID0.
    type: str
    choices: ['NoChange', 'UnconfiguredGood', 'Jbod']
    default: 'NoChange'
  default_drive_mode:
    description:
      - All unconfigured drives will move to the selected state on deployment.
      - Newly inserted drives will move to the selected state.
      - Select UnconfiguredGood option to retain the existing configuration.
      - Select Jbod to move the unconfigured drives to JBOD state.
      - Select RAID0 to create a RAID0 virtual drive on each of the unconfigured drives.
    type: str
    choices: ['UnconfiguredGood', 'Jbod', 'RAID0']
    default: 'UnconfiguredGood'
  secure_jbods:
    description:
      - JBOD drives specified in this slot range will be encrypted.
      - Allowed values are 'ALL', or a comma or hyphen separated number range.
      - Sample format is ALL or 1, 3 or 4-6, 8.
      - Setting the value to 'ALL' will encrypt all the unused UnconfigureGood/JBOD disks.
      - 'Slot format examples: "1,4,5", "2", "1-5", "1,2,6-8"'
    type: str
  m2_virtual_drive_config:
    description:
      - M.2 RAID virtual drive configuration.
    type: dict
    suboptions:
      enable:
        description:
          - Enable M.2 virtual drive configuration.
        type: bool
        default: false
      controller_slot:
        description:
          - Slot of the M.2 RAID controller for virtual drive creation.
          - Select 'MSTOR-RAID-1' to create virtual drive on the M.2 RAID controller in the first slot.
          - Select 'MSTOR-RAID-2' for second slot.
          - Select 'MSTOR-RAID-1,MSTOR-RAID-2' for both slots or either slot.
        type: str
        choices: ['MSTOR-RAID-2', 'MSTOR-RAID-1', 'MSTOR-RAID-1,MSTOR-RAID-2']
        default: 'MSTOR-RAID-1'
      name:
        description:
          - The name of the virtual drive.
          - The name can be between 1 and 15 alphanumeric characters.
          - Spaces or any special characters other than - (hyphen) and _ (underscore) are not allowed.
        type: str
        default: 'MStorBootVd'
  raid0_drive_config:
    description:
      - MRAID/RAID Single Drive RAID0 Configuration.
    type: dict
    suboptions:
      enable:
        description:
          - Enable RAID0 drive configuration.
        type: bool
        default: false
      drive_slots:
        description:
          - The set of drive slots where RAID0 virtual drives must be created.
          - If not specified, it will not be added.
          - 'Slot format examples: "1,4,5", "2", "1-5", "1,2,6-8"'
        type: str
      strip_size:
        description:
          - Desired strip size in KiB.
        type: int
        choices: [64, 128, 256, 512, 1024]
        default: 64
      access_policy:
        description:
          - Access policy that host has on this virtual drive.
        type: str
        choices: ['Default', 'ReadWrite', 'ReadOnly', 'Blocked']
        default: 'Default'
      read_policy:
        description:
          - Read ahead mode to be used to read data from this virtual drive.
        type: str
        choices: ['Default', 'ReadAhead', 'NoReadAhead']
        default: 'Default'
      write_policy:
        description:
          - Write mode to be used to write data to this virtual drive.
        type: str
        choices: ['Default', 'WriteThrough', 'WriteBackGoodBbu', 'AlwaysWriteBack']
        default: 'Default'
      disk_cache:
        description:
          - Disk cache policy for the virtual drive.
        type: str
        choices: ['Default', 'NoChange', 'Enable', 'Disable']
        default: 'Default'
  controller_attached_nvme_slots:
    description:
      - Only U.3 NVMe drives need to be specified, entered slots will be moved to controller attached mode.
      - Allowed slots are 1-9, 21-24, 101-104.
      - Allowed value is a comma or hyphen separated number ranges.
      - 'Slot format examples: "1,4,5", "2", "1-5", "1,2,6-8"'
    type: str
  direct_attached_nvme_slots:
    description:
      - Only U.3 NVMe drives need to be specified, entered slots will be moved to Direct attached mode.
      - Allowed slots are 1-9, 21-24, 101-104.
      - Allowed value is a comma or hyphen separated number ranges.
      - 'Slot format examples: "1,4,5", "2", "1-5", "1,2,6-8"'
    type: str
  drive_groups:
    description:
      - List of drive groups to be created and attached to the storage policy.
      - Each drive group can contain multiple virtual drives.
      - Leave empty to create a policy without drive groups for manual configuration later.
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - The name of the drive group.
          - The name can be between 1 and 64 alphanumeric characters.
        type: str
        required: true
      raid_level:
        description:
          - The supported RAID level for the disk group.
        type: str
        choices: ['Raid0', 'Raid1', 'Raid5', 'Raid6', 'Raid10', 'Raid50', 'Raid60']
        default: 'Raid0'
      secure_drive_group:
        description:
          - Enables/disables the drive security on all the drives used in this policy.
          - This flag just enables the drive security and only after Remote/Manual key setting configured, the actual security will be applied.
        type: bool
        default: false
      dedicated_hot_spares:
        description:
          - A collection of drives to be used as hot spares for this Drive Group.
          - Not applicable for RAID0.
          - 'Slot format examples: "1,4,5", "2", "1-5", "1,2,6-8"'
        type: str
      span_groups:
        description:
          - List of span groups for the drive group.
          - Each span group contains drive slots.
          - Required field - must be provided for all drive groups.
          - Single span group for RAID0, RAID1, RAID5, RAID6.
          - Multiple span groups (2-8) for RAID10, RAID50, RAID60.
        type: list
        elements: dict
        required: true
        suboptions:
          slots:
            description:
              - Drive slots for this span group.
              - 'Slot format examples: "1,4,5", "2", "1-5", "1,2,6-8"'
            type: str
            required: true
      virtual_drives:
        description:
          - List of virtual drives to be created in this drive group.
        type: list
        elements: dict
        suboptions:
          name:
            description:
              - The name of the virtual drive.
              - The name can be between 1 and 15 alphanumeric characters.
              - Spaces or any special characters other than - (hyphen) and _ (underscore) are not allowed.
            type: str
            required: true
          size:
            description:
              - Virtual drive size in MebiBytes.
              - Size is mandatory field except when the Expand to Available option is enabled.
            type: int
          expand_to_available:
            description:
              - Whether to expand the virtual drive to use all available space.
            type: bool
            default: false
          boot_drive:
            description:
              - Whether this virtual drive is a boot drive.
            type: bool
            default: false
          strip_size:
            description:
              - Desired strip size in KiB.
            type: int
            choices: [64, 128, 256, 512, 1024]
            default: 64
          access_policy:
            description:
              - Access policy that host has on this virtual drive.
            type: str
            choices: ['Default', 'ReadWrite', 'ReadOnly', 'Blocked']
            default: 'Default'
          read_policy:
            description:
              - Read ahead mode to be used to read data from this virtual drive.
            type: str
            choices: ['Default', 'ReadAhead', 'NoReadAhead']
            default: 'Default'
          write_policy:
            description:
              - Write mode to be used to write data to this virtual drive.
            type: str
            choices: ['Default', 'WriteThrough', 'WriteBackGoodBbu', 'AlwaysWriteBack']
            default: 'Default'
          disk_cache:
            description:
              - Disk cache policy for the virtual drive.
            type: str
            choices: ['Default', 'NoChange', 'Enable', 'Disable']
            default: 'Default'
      state:
        description:
          - Whether to create/update or delete the drive group.
        type: str
        choices: ['present', 'absent']
        default: present
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a basic storage policy
  cisco.intersight.intersight_storage_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "basic-storage-policy"
    description: "Basic storage policy configuration"
    m2_virtual_drive_config:
      enable: false
    raid0_drive_config:
      enable: false
    tags:
      - Key: "Environment"
        Value: "Production"

- name: Create storage policy with M.2 virtual drive enabled
  cisco.intersight.intersight_storage_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "m2-enabled-storage-policy"
    description: "Storage policy with M.2 virtual drive"
    m2_virtual_drive_config:
      enable: true
      controller_slot: "MSTOR-RAID-1"
      name: "MStorBootVd"
    raid0_drive_config:
      enable: false
    tags:
      - Key: "Site"
        Value: "Datacenter1"

- name: Create storage policy with RAID0 configuration
  cisco.intersight.intersight_storage_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "raid0-storage-policy"
    description: "Storage policy with RAID0 configuration"
    use_jbod_for_vd_creation: true
    default_drive_mode: "RAID0"
    secure_jbods: "1"
    m2_virtual_drive_config:
      enable: false
    raid0_drive_config:
      enable: true
      strip_size: 128
      access_policy: "ReadWrite"
      read_policy: "ReadAhead"
      write_policy: "WriteBackGoodBbu"
      disk_cache: "Enable"
    controller_attached_nvme_slots: "2"
    direct_attached_nvme_slots: "3"

- name: Create storage policy with drive groups
  cisco.intersight.intersight_storage_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "drive-group-storage-policy"
    description: "Storage policy with drive groups"
    m2_virtual_drive_config:
      enable: false
    raid0_drive_config:
      enable: false
    drive_groups:
      - name: "raid0-group"
        raid_level: "Raid0"
        secure_drive_group: false
        span_groups:
          - slots: "1,2"
        virtual_drives:
          - name: "raid0-vd"
            size: 1024
            expand_to_available: false
            boot_drive: true
            strip_size: 128
            access_policy: "ReadWrite"
            read_policy: "ReadAhead"
            write_policy: "WriteBackGoodBbu"
            disk_cache: "Enable"
      - name: "raid1-group"
        raid_level: "Raid1"
        secure_drive_group: true
        dedicated_hot_spares: "3,4"
        span_groups:
          - slots: "5,6"
        virtual_drives:
          - name: "raid1-vd"
            size: 0
            expand_to_available: true
            boot_drive: false
            strip_size: 64
            access_policy: "Default"
            read_policy: "Default"
            write_policy: "Default"
            disk_cache: "Default"

- name: Create storage policy with nested RAID drive groups
  cisco.intersight.intersight_storage_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "nested-raid-storage-policy"
    description: "Storage policy with nested RAID configurations"
    m2_virtual_drive_config:
      enable: false
    raid0_drive_config:
      enable: false
    drive_groups:
      - name: "raid60-group"
        raid_level: "Raid60"
        secure_drive_group: false
        dedicated_hot_spares: "33,34,35,36"
        span_groups:
          - slots: "1,2,3,4"
          - slots: "5,6,7,8"
          - slots: "9,10,11,12"
          - slots: "13,14,15,16"
          - slots: "17,18,19,20"
          - slots: "21,22,23,24"
          - slots: "25,26,27,28"
          - slots: "29,30,31,32"
        virtual_drives:
          - name: "raid60-vd1"
            size: 1024
            expand_to_available: false
            boot_drive: true
            strip_size: 64
            access_policy: "ReadWrite"
            read_policy: "Default"
            write_policy: "Default"
            disk_cache: "Default"
          - name: "raid60-vd2"
            size: 1024
            expand_to_available: false
            boot_drive: false
            strip_size: 64
            access_policy: "ReadWrite"
            read_policy: "Default"
            write_policy: "Default"
            disk_cache: "Default"

- name: Delete a storage policy
  cisco.intersight.intersight_storage_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "old-storage-policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "basic-storage-policy",
        "UseJbodForVdCreation": true,
        "UnusedDisksState": "NoChange",
        "DefaultDriveMode": "UnconfiguredGood",
        "M2VirtualDrive": {
            "Enable": false,
            "ControllerSlot": "MSTOR-RAID-1",
            "Name": "MStorBootVd"
        },
        "Raid0Drive": {
            "Enable": false,
            "VirtualDrivePolicy": {
                "StripSize": 64,
                "AccessPolicy": "Default",
                "ReadPolicy": "Default",
                "WritePolicy": "Default",
                "DriveCache": "Default"
            }
        },
        "ObjectType": "storage.StoragePolicy",
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


# Virtual drive policy specification
VIRTUAL_DRIVE_POLICY_SPEC = {
    'strip_size': dict(type='int', choices=[64, 128, 256, 512, 1024], default=64),
    'access_policy': dict(type='str', choices=['Default', 'ReadWrite', 'ReadOnly', 'Blocked'], default='Default'),
    'read_policy': dict(type='str', choices=['Default', 'ReadAhead', 'NoReadAhead'], default='Default'),
    'write_policy': dict(type='str', choices=['Default', 'WriteThrough', 'WriteBackGoodBbu', 'AlwaysWriteBack'], default='Default'),
    'disk_cache': dict(type='str', choices=['Default', 'NoChange', 'Enable', 'Disable'], default='Default')
}


def build_virtual_drive_policy(virtual_drive_config):
    """
    Build VirtualDrivePolicy from virtual drive configuration.
    Args:
        virtual_drive_config: Dictionary containing virtual drive policy parameters
    Returns:
        Dictionary representing VirtualDrivePolicy
    """
    return {
        'StripSize': virtual_drive_config.get('strip_size', 64),
        'AccessPolicy': virtual_drive_config.get('access_policy', 'Default'),
        'ReadPolicy': virtual_drive_config.get('read_policy', 'Default'),
        'WritePolicy': virtual_drive_config.get('write_policy', 'Default'),
        'DriveCache': virtual_drive_config.get('disk_cache', 'Default')
    }


def validate_input(module):
    """
    Validate input parameters for the storage policy module.
    """
    # Validate UseJbodForVdCreation vs DefaultDriveMode compatibility
    use_jbod = module.params.get('use_jbod_for_vd_creation')
    default_drive_mode = module.params.get('default_drive_mode')
    unused_disks_state = module.params.get('unused_disks_state')

    # UseJbodForVdCreation must be disabled if DefaultDriveMode is JBOD
    if default_drive_mode == 'Jbod' and use_jbod:
        module.fail_json(msg="use_jbod_for_vd_creation must be disabled when default_drive_mode is set to 'Jbod'")

    # UnusedDisksState must be NoChange if DefaultDriveMode is JBOD or RAID0
    if default_drive_mode in ['Jbod', 'RAID0'] and unused_disks_state != 'NoChange':
        module.fail_json(msg="unused_disks_state must be set to 'NoChange' when default_drive_mode is 'Jbod' or 'RAID0'")

    # Validate M.2 virtual drive configuration
    m2_config = module.params.get('m2_virtual_drive_config')
    if m2_config and m2_config.get('enable'):
        if not m2_config.get('controller_slot'):
            module.fail_json(msg="controller_slot is required when M.2 virtual drive is enabled")
        if not m2_config.get('name'):
            module.fail_json(msg="name is required when M.2 virtual drive is enabled")

        # Validate name format
        name = m2_config.get('name')
        if len(name) > 15 or len(name) < 1:
            module.fail_json(msg="M.2 virtual drive name must be between 1 and 15 characters")

    # Validate drive groups configuration
    drive_groups = module.params.get('drive_groups', [])
    if drive_groups:
        for drive_group in drive_groups:
            # Validate drive group name
            name = drive_group.get('name', '')
            if len(name) > 64 or len(name) < 1:
                module.fail_json(msg="Drive group name must be between 1 and 64 characters")

            # Validate dedicated hot spares for RAID0
            raid_level = drive_group.get('raid_level')
            dedicated_hot_spares = drive_group.get('dedicated_hot_spares')
            if raid_level == 'Raid0' and dedicated_hot_spares:
                module.fail_json(msg="Dedicated hot spares are not applicable for RAID0")

            # Validate span groups requirements based on RAID level
            span_groups = drive_group.get('span_groups', [])
            if not span_groups:
                module.fail_json(msg="span_groups is required for drive groups")

            # Validate span group count based on RAID level
            span_count = len(span_groups)
            if raid_level in ['Raid10', 'Raid50', 'Raid60']:
                if span_count > 8:
                    module.fail_json(msg=f"RAID level {raid_level} supports maximum 8 span groups, got {span_count}")
                if span_count < 2:
                    module.fail_json(msg=f"RAID level {raid_level} requires minimum 2 span groups, got {span_count}")
            elif raid_level in ['Raid0', 'Raid1', 'Raid5', 'Raid6']:
                if span_count != 1:
                    module.fail_json(msg=f"RAID level {raid_level} requires exactly 1 span group, got {span_count}")

            # Validate virtual drives
            virtual_drives = drive_group.get('virtual_drives', [])
            for virtual_drive in virtual_drives:
                # Validate virtual drive name
                vd_name = virtual_drive.get('name', '')
                if len(vd_name) > 15 or len(vd_name) < 1:
                    module.fail_json(msg="Virtual drive name must be between 1 and 15 characters")

                # Validate size requirement
                size = virtual_drive.get('size')
                expand_to_available = virtual_drive.get('expand_to_available', False)
                if not expand_to_available and not size:
                    module.fail_json(msg="Virtual drive size is mandatory except when expand_to_available is enabled")


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict', default=[]),
        use_jbod_for_vd_creation=dict(type='bool', default=False),
        unused_disks_state=dict(type='str', choices=['NoChange', 'UnconfiguredGood', 'Jbod'], default='NoChange'),
        default_drive_mode=dict(type='str', choices=['UnconfiguredGood', 'Jbod', 'RAID0'], default='UnconfiguredGood'),
        secure_jbods=dict(type='str'),
        m2_virtual_drive_config=dict(
            type='dict',
            required=False,
            options=dict(
                enable=dict(type='bool', default=False),
                controller_slot=dict(type='str',
                                     choices=['MSTOR-RAID-2', 'MSTOR-RAID-1', 'MSTOR-RAID-1,MSTOR-RAID-2'],
                                     default='MSTOR-RAID-1'),
                name=dict(type='str', default='MStorBootVd')
            ),
        ),
        raid0_drive_config=dict(
            type='dict',
            required=False,
            options=dict(
                enable=dict(type='bool', default=False),
                drive_slots=dict(type='str'),
                strip_size=VIRTUAL_DRIVE_POLICY_SPEC['strip_size'],
                access_policy=VIRTUAL_DRIVE_POLICY_SPEC['access_policy'],
                read_policy=VIRTUAL_DRIVE_POLICY_SPEC['read_policy'],
                write_policy=VIRTUAL_DRIVE_POLICY_SPEC['write_policy'],
                disk_cache=VIRTUAL_DRIVE_POLICY_SPEC['disk_cache']
            ),
        ),
        controller_attached_nvme_slots=dict(type='str'),
        direct_attached_nvme_slots=dict(type='str'),
        drive_groups=dict(
            type='list',
            elements='dict',
            options=dict(
                name=dict(type='str', required=True),
                raid_level=dict(type='str', choices=['Raid0', 'Raid1', 'Raid5', 'Raid6', 'Raid10', 'Raid50', 'Raid60'], default='Raid0'),
                secure_drive_group=dict(type='bool', default=False),
                dedicated_hot_spares=dict(type='str'),
                span_groups=dict(
                    type='list',
                    elements='dict',
                    required=True,
                    options=dict(
                        slots=dict(type='str', required=True)
                    )
                ),
                virtual_drives=dict(
                    type='list',
                    elements='dict',
                    options=dict(
                        name=dict(type='str', required=True),
                        size=dict(type='int'),
                        expand_to_available=dict(type='bool', default=False),
                        boot_drive=dict(type='bool', default=False),
                        strip_size=VIRTUAL_DRIVE_POLICY_SPEC['strip_size'],
                        access_policy=VIRTUAL_DRIVE_POLICY_SPEC['access_policy'],
                        read_policy=VIRTUAL_DRIVE_POLICY_SPEC['read_policy'],
                        write_policy=VIRTUAL_DRIVE_POLICY_SPEC['write_policy'],
                        disk_cache=VIRTUAL_DRIVE_POLICY_SPEC['disk_cache']
                    )
                ),
                state=dict(type='str', choices=['present', 'absent'], default='present')
            )
        ),
    )

    required_if = [
        ['state', 'present', ['m2_virtual_drive_config', 'raid0_drive_config']],
    ]

    module = AnsibleModule(
        argument_spec,
        required_if=required_if,
        supports_check_mode=True,
    )

    if module.params['state'] == 'present':
        validate_input(module)

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/storage/StoragePolicies'

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }

    if module.params['state'] == 'present':
        intersight.set_tags_and_description()

        # Set basic storage configuration
        intersight.api_body['UseJbodForVdCreation'] = intersight.module.params['use_jbod_for_vd_creation']
        intersight.api_body['UnusedDisksState'] = intersight.module.params['unused_disks_state']
        intersight.api_body['DefaultDriveMode'] = intersight.module.params['default_drive_mode']

        # Set secure JBODs if specified
        if module.params['secure_jbods']:
            intersight.api_body['SecureJbods'] = module.params['secure_jbods']

        # Set M.2 virtual drive configuration
        m2_config = module.params.get('m2_virtual_drive_config', {})
        intersight.api_body['M2VirtualDrive'] = {
            'Enable': m2_config.get('enable', False)
        }
        if m2_config.get('enable'):
            intersight.api_body['M2VirtualDrive']['ControllerSlot'] = m2_config.get('controller_slot', 'MSTOR-RAID-1')
            intersight.api_body['M2VirtualDrive']['Name'] = m2_config.get('name', 'MStorBootVd')

        # Set RAID0 drive configuration
        raid0_config = module.params.get('raid0_drive_config', {})
        intersight.api_body['Raid0Drive'] = {
            'Enable': raid0_config.get('enable', False)
        }
        if raid0_config.get('enable'):
            if raid0_config.get('drive_slots'):
                intersight.api_body['Raid0Drive']['DriveSlots'] = raid0_config['drive_slots']

            # Set virtual drive policy using the common function
            intersight.api_body['Raid0Drive']['VirtualDrivePolicy'] = build_virtual_drive_policy(raid0_config)

        # Set NVMe slot configurations
        if module.params['controller_attached_nvme_slots']:
            intersight.api_body['ControllerAttachedNvmeSlots'] = module.params['controller_attached_nvme_slots']

        if module.params['direct_attached_nvme_slots']:
            intersight.api_body['DirectAttachedNvmeSlots'] = module.params['direct_attached_nvme_slots']

    intersight.configure_policy_or_profile(resource_path)

    # Save the storage policy response
    storage_policy_response = intersight.result['api_response']
    storage_policy_moid = None
    if module.params['state'] == 'present' and storage_policy_response:
        storage_policy_moid = storage_policy_response.get('Moid')

    # Process drive groups if provided
    drive_groups_response = []
    if module.params['state'] == 'present' and module.params['drive_groups']:
        for drive_group_config in module.params['drive_groups']:
            # Build drive group API body
            drive_group_api_body = {
                'Name': drive_group_config['name'],
                'RaidLevel': drive_group_config['raid_level'],
                'SecureDriveGroup': drive_group_config.get('secure_drive_group', False),
                'Type': 0,  # Manual drive group
                'StoragePolicy': storage_policy_moid
            }

            # Build ManualDriveGroup
            manual_drive_group = {}

            # Add dedicated hot spares if specified (not for RAID0)
            dedicated_hot_spares = drive_group_config.get('dedicated_hot_spares')
            if dedicated_hot_spares and drive_group_config['raid_level'] != 'Raid0':
                manual_drive_group['DedicatedHotSpares'] = dedicated_hot_spares

            # Build span groups (required field)
            span_groups = drive_group_config.get('span_groups', [])
            # Transform lowercase 'slots' to uppercase 'Slots' for API
            api_span_groups = []
            for span_group in span_groups:
                api_span_groups.append({'Slots': span_group['slots']})
            manual_drive_group['SpanGroups'] = api_span_groups

            drive_group_api_body['ManualDriveGroup'] = manual_drive_group

            # Build virtual drives
            virtual_drives = []
            for virtual_drive_config in drive_group_config.get('virtual_drives', []):
                virtual_drive = {
                    'Name': virtual_drive_config['name'],
                    'ExpandToAvailable': virtual_drive_config.get('expand_to_available', False),
                    'BootDrive': virtual_drive_config.get('boot_drive', False),
                    'VirtualDrivePolicy': build_virtual_drive_policy(virtual_drive_config)
                }

                # Add size if specified and not expanding to available
                if not virtual_drive_config.get('expand_to_available', False):
                    virtual_drive['Size'] = virtual_drive_config.get('size', 0)

                virtual_drives.append(virtual_drive)

            drive_group_api_body['VirtualDrives'] = virtual_drives

            # Create the drive group
            resource_path = '/storage/DriveGroups'
            intersight.api_body = drive_group_api_body
            # Filter by both DriveGroup name AND StoragePolicy to avoid affecting DriveGroups in other policies
            custom_filter = f"Name eq '{drive_group_config['name']}' and StoragePolicy.Moid eq '{storage_policy_moid}'"
            intersight.configure_secondary_resource(
                resource_path=resource_path,
                state=drive_group_config.get('state', 'present'),
                custom_filter=custom_filter
            )

            # Save the drive group response
            drive_groups_response.append(intersight.result['api_response'])

    # Combine storage policy and drive groups in the main response
    if storage_policy_response:
        storage_policy_response['DriveGroups'] = drive_groups_response
        intersight.result['api_response'] = storage_policy_response

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: jenkins_build_info
short_description: Get information about Jenkins builds
version_added: 7.4.0
description:
  - Get information about Jenkins builds with Jenkins REST API.
requirements:
  - "python-jenkins >= 0.4.12"
author:
  - Juan Casanova (@juanmcasanova)
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.info_module
options:
  name:
    description:
      - Name of the Jenkins job to which the build belongs.
    required: true
    type: str
  build_number:
    description:
      - An integer which specifies a build of a job.
      - If not specified the last build information is returned.
    type: int
  password:
    description:
      - Password to authenticate with the Jenkins server.
    type: str
  token:
    description:
      - API token used to authenticate with the Jenkins server.
    type: str
  url:
    description:
      - URL of the Jenkins server.
    default: http://localhost:8080
    type: str
  user:
    description:
      - User to authenticate with the Jenkins server.
    type: str
"""

EXAMPLES = r"""
- name: Get information about a jenkins build using basic authentication
  community.general.jenkins_build_info:
    name: "test-check"
    build_number: 1
    user: admin
    password: asdfg
    url: http://localhost:8080

- name: Get information about a jenkins build anonymously
  community.general.jenkins_build_info:
    name: "stop-check"
    build_number: 3
    url: http://localhost:8080

- name: Get information about a jenkins build using token authentication
  community.general.jenkins_build_info:
    name: "delete-experiment"
    build_number: 30
    user: Jenkins
    token: abcdefghijklmnopqrstuvwxyz123456
    url: http://localhost:8080
"""

RETURN = r"""
name:
  description: Name of the jenkins job.
  returned: success
  type: str
  sample: "test-job"
state:
  description: State of the jenkins job.
  returned: success
  type: str
  sample: present
user:
  description: User used for authentication.
  returned: success
  type: str
  sample: admin
url:
  description: URL to connect to the Jenkins server.
  returned: success
  type: str
  sample: https://jenkins.mydomain.com
build_info:
  description: Build info of the jenkins job.
  returned: success
  type: dict
"""

import traceback

JENKINS_IMP_ERR = None
try:
    import jenkins

    python_jenkins_installed = True
except ImportError:
    JENKINS_IMP_ERR = traceback.format_exc()
    python_jenkins_installed = False

from ansible.module_utils.basic import AnsibleModule, missing_required_lib


class JenkinsBuildInfo:
    def __init__(self, module):
        self.module = module

        self.name = module.params.get("name")
        self.password = module.params.get("password")
        self.token = module.params.get("token")
        self.user = module.params.get("user")
        self.jenkins_url = module.params.get("url")
        self.build_number = module.params.get("build_number")
        self.server = self.get_jenkins_connection()

        self.result = {
            "changed": False,
            "url": self.jenkins_url,
            "name": self.name,
            "user": self.user,
        }

    def get_jenkins_connection(self):
        try:
            if self.user and self.password:
                return jenkins.Jenkins(self.jenkins_url, self.user, self.password)
            elif self.user and self.token:
                return jenkins.Jenkins(self.jenkins_url, self.user, self.token)
            elif self.user and not (self.password or self.token):
                return jenkins.Jenkins(self.jenkins_url, self.user)
            else:
                return jenkins.Jenkins(self.jenkins_url)
        except Exception as e:
            self.module.fail_json(msg=f"Unable to connect to Jenkins server, {e}")

    def get_build_status(self):
        try:
            if self.build_number is None:
                job_info = self.server.get_job_info(self.name)
                self.build_number = job_info["lastBuild"]["number"]

            return self.server.get_build_info(self.name, self.build_number)
        except jenkins.JenkinsException:
            response = {}
            response["result"] = "ABSENT"
            return response
        except Exception as e:
            self.module.fail_json(msg=f"Unable to fetch build information, {e}", exception=traceback.format_exc())

    def get_result(self):
        result = self.result
        build_status = self.get_build_status()

        if build_status["result"] == "ABSENT":
            result["failed"] = True
        result["build_info"] = build_status

        return result


def test_dependencies(module):
    if not python_jenkins_installed:
        module.fail_json(
            msg=missing_required_lib(
                "python-jenkins", url="https://python-jenkins.readthedocs.io/en/latest/install.html"
            ),
            exception=JENKINS_IMP_ERR,
        )


def main():
    module = AnsibleModule(
        argument_spec=dict(
            build_number=dict(type="int"),
            name=dict(required=True),
            password=dict(no_log=True),
            token=dict(no_log=True),
            url=dict(default="http://localhost:8080"),
            user=dict(),
        ),
        mutually_exclusive=[["password", "token"]],
        supports_check_mode=True,
    )

    test_dependencies(module)
    jenkins_build_info = JenkinsBuildInfo(module)

    result = jenkins_build_info.get_result()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The iosxr_ospfv2 class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.facts.facts import Facts
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.rm_templates.ospfv2 import (
    Ospfv2Template,
)


class Ospfv2(ResourceModule):
    """
    The ios_ospfv2 class
    """

    gather_subset = ["!all", "!min"]

    gather_network_resources = ["ospfv2"]

    def __init__(self, module):
        super(Ospfv2, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="ospfv2",
            tmplt=Ospfv2Template(),
        )

    def execute_module(self):
        """Execute the module
        :rtype: A dictionary
        :returns: The result from module execution
        """
        self.gen_config()
        self.run_commands()

        return self.result

    def _remove_false_booleans(self, config_dict):
        """Recursively remove boolean fields set to False and empty dictionaries.

        IOS-XR commands only need to specify boolean options when they are True.
        False booleans should not appear in the generated configuration.
        After removing False values, empty dictionaries are also removed.

        :param config_dict: Dictionary to clean
        """
        keys_to_remove = []
        for key, value in config_dict.items():
            if isinstance(value, bool) and value is False:
                keys_to_remove.append(key)
            elif isinstance(value, dict):
                self._remove_false_booleans(value)
                # After recursively cleaning, remove if dictionary is now empty
                if not value:
                    keys_to_remove.append(key)

        for key in keys_to_remove:
            config_dict.pop(key)

    def _clean_max_metric(self, process_dict):
        """Clean up mutually exclusive max_metric configuration options.
        Recursively check every key, and if any boolean is set to False
        as set: False, pop them since they are not required to be set.
        Handles two types of conflicts in max_metric configuration:
        1. summary_lsa/external_lsa: 'set' (uses default IOS-XR value) vs
           'max_metric_value' (explicit value) - only one can be specified.
           When both are present, 'set' is removed in favor of the explicit value.
        2. on_startup: 'wait_period' vs 'wait_for_bgp' - mutually exclusive options.
           When both are present, 'wait_period' is removed.

        :param process_dict: Dictionary of OSPF processes with max_metric configuration
        """
        for process in process_dict:
            max_metric = process_dict[process].get("max_metric", {})
            router_lsa = max_metric.get("router_lsa", {})

            if not router_lsa:
                continue

            # Remove any False boolean values recursively and clean up empty dicts
            self._remove_false_booleans(router_lsa)

            router_lsa["set"] = True

            summary_lsa = router_lsa.get("summary_lsa", {})
            if summary_lsa.get("set") and summary_lsa.get("max_metric_value"):
                summary_lsa.pop("set")

            external_lsa = router_lsa.get("external_lsa", {})
            if external_lsa.get("set") and external_lsa.get("max_metric_value"):
                external_lsa.pop("set")

            on_startup = router_lsa.get("on_startup", {})
            if on_startup.get("wait_period") and on_startup.get("wait_for_bgp"):
                on_startup.pop("wait_period")

    def gen_config(self):
        """Select the appropriate function based on the state provided
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        if self.want:
            wantd = {
                (entry["process_id"], entry.get("vrf")): entry
                for entry in self.want.get("processes", [])
            }
        else:
            wantd = {}
        if self.have:
            haved = {
                (entry["process_id"], entry.get("vrf")): entry
                for entry in self.have.get("processes", [])
            }
        else:
            haved = {}

        self._clean_max_metric(wantd)

        # turn all lists of dicts into dicts prior to merge
        for thing in wantd, haved:
            for _pid, proc in thing.items():
                for area in proc.get("areas", []):
                    virtual_link = {entry["id"]: entry for entry in area.get("virtual_link", [])}
                    if bool(virtual_link):
                        area["virtual_link"] = virtual_link
                    ranges = {entry["address"]: entry for entry in area.get("ranges", [])}
                    if bool(ranges):
                        area["ranges"] = ranges

                proc["areas"] = {entry["area_id"]: entry for entry in proc.get("areas", [])}
                if proc.get("distribute_list"):
                    if "acls" in proc.get("distribute_list"):
                        proc["distribute_list"]["acls"] = {
                            entry["name"]: entry
                            for entry in proc["distribute_list"].get(
                                "acls",
                                [],
                            )
                        }

        # if state is merged, merge want onto have
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, limit the have to anything in want
        # set want to nothing
        if self.state == "deleted":
            haved = {k: v for k, v in haved.items() if k in wantd or not wantd}
            wantd = {}

        # delete processes first so we do run into "more than one" errors
        if self.state == "deleted":
            haved_del = deepcopy(haved)
            want_process = {}
            for k, t_want in haved_del.items():
                want_process["process_id"] = t_want.get("process_id")
                if not (len(t_want) == 2 and not t_want.get("areas")):
                    self._compare(want=want_process, have=haved_del.get(k, {}))
        if self.state == "overridden":
            haved_del = deepcopy(haved)
            want = {}
            for k, t_want in haved_del.items():
                if k not in wantd:
                    want["process_id"] = t_want.get("process_id")
                    if not (len(t_want) == 2 and not t_want.get("areas")):
                        self._compare(want=want, have=haved_del.get(k, {}))

        for k, want in wantd.items():
            self._compare(want=want, have=haved.pop(k, {}))

    def _compare(self, want, have):
        parsers = [
            "bfd",
            "cost",
            "weight",
            "passive",
            "priority",
            "protocol",
            "auto_cost",
            "bandwidth",
            "flood_reduction",
            "default_metric",
            "default_weight",
            "router_id",
            "demand_circuit",
            "packet_size",
            "transmit_delay",
            "summary_in",
            "external_out",
            "dead_interval",
            "hello_interval",
            "authentication",
            "adjacency_stagger",
            "retransmit_interval",
            "mtu_ignore",
            "bfd.fast_detect",
            "capability",
            "capability.opaque",
            "admin_distance",
            "ospf_distance",
            "address_family_unicast",
            "loopback_stub_network",
            "authentication.message_digest",
            "default_information_originate",
            "link_down_fast_detect",
            "nsr",
            "database_filter",
            "log_adjacency_changes",
            "distribute_bgp_ls",
            "distribute_link_state",
            "max_lsa",
            "max_metric",
            "mpls_ldp",
            "mpls_traffic_eng",
            "microloop_avoidance",
            "prefix_suppression",
            "protocol_shutdown",
            "timers.lsa",
            "timers.graceful_shutdown",
            "throttle.lsa_all",
            "throttle.spf",
            "throttle.fast_reroute",
            "timers.pacing_flood",
        ]

        if want != have:
            self.addcmd(want or have, "pid", False)
            self.compare(parsers, want, have)
            self._areas_compare(want, have)

    def _areas_compare(self, want, have):
        wareas = want.get("areas", {})
        hareas = have.get("areas", {})
        for name, entry in wareas.items():
            self._area_compare(want=entry, have=hareas.pop(name, {}))
        for name, entry in hareas.items():
            self._area_compare(want={}, have=entry)

    def _area_compare(self, want, have):
        parsers = [
            "area.authentication",
            "area.authentication_key",
            "area.authentication.message_digest",
            "area.mpls_traffic_eng",
            "area.mpls_ldp",
            "area.bfd",
            "area.bfd.fast_detect",
            "area.nssa",
            "area.nssa.default_information_originate",
            "area.nssa.translate",
            "area.default_cost",
            "area.stub",
            "area.ranges",
            "area.cost",
            "area.dead_interval",
            "area.hello_interval",
            "area.transmit_delay",
            "area.mtu_ignore",
            "area.packet_size",
            "area.priority",
            "area.weight",
            "area.external_out",
            "area.summary_in",
            "area.demand_circuit",
            "area.passive",
        ]
        self.compare(parsers=parsers, want=want, have=have)
        self._areas_compare_virtual_link(want, have)

    def _areas_compare_virtual_link(self, want, have):
        wvlinks = want.get("virtual_link", {})
        hvlinks = have.get("virtual_link", {})
        for name, entry in wvlinks.items():
            self._area_compare_virtual_link(
                want=entry,
                have=hvlinks.pop(name, {}),
            )
        for name, entry in hvlinks.items():
            self._area_compare_virtual_link(want={}, have=entry)

    def _area_compare_virtual_link(self, want, have):
        parsers = [
            "virtual_link.authentication",
            "virtual_link.authentication_key",
            "virtual_link.authentication.message_digest",
            "virtual_link.hello_interval",
            "virtual_link.dead_interval",
            "virtual_link.retransmit_interval",
        ]
        self.compare(parsers=parsers, want=want, have=have)



# Copyright (c) 2017, Brian Coca
# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Make coding more python3-ish
from __future__ import annotations

DOCUMENTATION = r"""
name: pickle
short_description: Pickle formatted files
description:
  - This cache uses Python's pickle serialization format, in per host files, saved to the filesystem.
author: Brian Coca (@bcoca)
options:
  _uri:
    required: true
    description:
      - Path in which the cache plugin saves the files.
    env:
      - name: ANSIBLE_CACHE_PLUGIN_CONNECTION
    ini:
      - key: fact_caching_connection
        section: defaults
    type: path
  _prefix:
    description: User defined prefix to use when creating the files.
    env:
      - name: ANSIBLE_CACHE_PLUGIN_PREFIX
    ini:
      - key: fact_caching_prefix
        section: defaults
    type: string
  _timeout:
    default: 86400
    description: Expiration timeout in seconds for the cache plugin data. Set to 0 to never expire.
    env:
      - name: ANSIBLE_CACHE_PLUGIN_TIMEOUT
    ini:
      - key: fact_caching_timeout
        section: defaults
    type: float
"""

import pickle

from ansible.plugins.cache import BaseFileCacheModule


class CacheModule(BaseFileCacheModule):
    """
    A caching module backed by pickle files.
    """

    _persistent = False  # prevent unnecessary JSON serialization and key munging

    def _load(self, filepath):
        # Pickle is a binary format
        with open(filepath, "rb") as f:
            return pickle.load(f, encoding="bytes")

    def _dump(self, value, filepath):
        with open(filepath, "wb") as f:
            # Use pickle protocol 2 which is compatible with Python 2.3+.
            pickle.dump(value, f, protocol=2)



#!/usr/bin/python

# Copyright (c) 2020, FERREIRA Christophe <christophe.ferreira@cnaf.fr>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: nomad_job_info
author: FERREIRA Christophe (@chris93111)
version_added: "1.3.0"
short_description: Get Nomad Jobs info
description:
  - Get info for one Nomad job.
  - List Nomad jobs.
requirements:
  - python-nomad
extends_documentation_fragment:
  - community.general._nomad
  - community.general._attributes
  - community.general._attributes.info_module
options:
  name:
    description:
      - Name of job for Get info.
      - If not specified, lists all jobs.
    type: str
seealso:
  - name: Nomad jobs documentation
    description: Complete documentation for Nomad API jobs.
    link: https://www.nomadproject.io/api-docs/jobs/
"""

EXAMPLES = r"""
- name: Get info for job awx
  community.general.nomad_job_info:
    host: localhost
    name: awx
  register: result

- name: List Nomad jobs
  community.general.nomad_job_info:
    host: localhost
  register: result
"""

RETURN = r"""
result:
  description: List with dictionary contains jobs info.
  returned: success
  type: list
  sample:
    [
      {
        "Affinities": null,
        "AllAtOnce": false,
        "Constraints": null,
        "ConsulToken": "",
        "CreateIndex": 13,
        "Datacenters": [
          "dc1"
        ],
        "Dispatched": false,
        "ID": "example",
        "JobModifyIndex": 13,
        "Meta": null,
        "ModifyIndex": 13,
        "Multiregion": null,
        "Name": "example",
        "Namespace": "default",
        "NomadTokenID": "",
        "ParameterizedJob": null,
        "ParentID": "",
        "Payload": null,
        "Periodic": null,
        "Priority": 50,
        "Region": "global",
        "Spreads": null,
        "Stable": false,
        "Status": "pending",
        "StatusDescription": "",
        "Stop": false,
        "SubmitTime": 1602244370615307000,
        "TaskGroups": [
          {
            "Affinities": null,
            "Constraints": null,
            "Count": 1,
            "EphemeralDisk": {
              "Migrate": false,
              "SizeMB": 300,
              "Sticky": false
            },
            "Meta": null,
            "Migrate": {
              "HealthCheck": "checks",
              "HealthyDeadline": 300000000000,
              "MaxParallel": 1,
              "MinHealthyTime": 10000000000
            },
            "Name": "cache",
            "Networks": null,
            "ReschedulePolicy": {
              "Attempts": 0,
              "Delay": 30000000000,
              "DelayFunction": "exponential",
              "Interval": 0,
              "MaxDelay": 3600000000000,
              "Unlimited": true
            },
            "RestartPolicy": {
              "Attempts": 3,
              "Delay": 15000000000,
              "Interval": 1800000000000,
              "Mode": "fail"
            },
            "Scaling": null,
            "Services": null,
            "ShutdownDelay": null,
            "Spreads": null,
            "StopAfterClientDisconnect": null,
            "Tasks": [
              {
                "Affinities": null,
                "Artifacts": null,
                "CSIPluginConfig": null,
                "Config": {
                  "image": "redis:3.2",
                  "port_map": [
                    {
                      "db": 6379.0
                    }
                  ]
                },
                "Constraints": null,
                "DispatchPayload": null,
                "Driver": "docker",
                "Env": null,
                "KillSignal": "",
                "KillTimeout": 5000000000,
                "Kind": "",
                "Leader": false,
                "Lifecycle": null,
                "LogConfig": {
                  "MaxFileSizeMB": 10,
                  "MaxFiles": 10
                },
                "Meta": null,
                "Name": "redis",
                "Resources": {
                  "CPU": 500,
                  "Devices": null,
                  "DiskMB": 0,
                  "IOPS": 0,
                  "MemoryMB": 256,
                  "Networks": [
                    {
                      "CIDR": "",
                      "DNS": null,
                      "Device": "",
                      "DynamicPorts": [
                        {
                          "HostNetwork": "default",
                          "Label": "db",
                          "To": 0,
                          "Value": 0
                        }
                      ],
                      "IP": "",
                      "MBits": 10,
                      "Mode": "",
                      "ReservedPorts": null
                    }
                  ]
                },
                "RestartPolicy": {
                  "Attempts": 3,
                  "Delay": 15000000000,
                  "Interval": 1800000000000,
                  "Mode": "fail"
                },
                "Services": [
                  {
                    "AddressMode": "auto",
                    "CanaryMeta": null,
                    "CanaryTags": null,
                    "Checks": [
                      {
                        "AddressMode": "",
                        "Args": null,
                        "CheckRestart": null,
                        "Command": "",
                        "Expose": false,
                        "FailuresBeforeCritical": 0,
                        "GRPCService": "",
                        "GRPCUseTLS": false,
                        "Header": null,
                        "InitialStatus": "",
                        "Interval": 10000000000,
                        "Method": "",
                        "Name": "alive",
                        "Path": "",
                        "PortLabel": "",
                        "Protocol": "",
                        "SuccessBeforePassing": 0,
                        "TLSSkipVerify": false,
                        "TaskName": "",
                        "Timeout": 2000000000,
                        "Type": "tcp"
                      }
                    ],
                    "Connect": null,
                    "EnableTagOverride": false,
                    "Meta": null,
                    "Name": "redis-cache",
                    "PortLabel": "db",
                    "Tags": [
                      "global",
                      "cache"
                    ],
                    "TaskName": ""
                  }
                ],
                "ShutdownDelay": 0,
                "Templates": null,
                "User": "",
                "Vault": null,
                "VolumeMounts": null
              }
            ],
            "Update": {
              "AutoPromote": false,
              "AutoRevert": false,
              "Canary": 0,
              "HealthCheck": "checks",
              "HealthyDeadline": 180000000000,
              "MaxParallel": 1,
              "MinHealthyTime": 10000000000,
              "ProgressDeadline": 600000000000,
              "Stagger": 30000000000
            },
            "Volumes": null
          }
        ],
        "Type": "service",
        "Update": {
          "AutoPromote": false,
          "AutoRevert": false,
          "Canary": 0,
          "HealthCheck": "",
          "HealthyDeadline": 0,
          "MaxParallel": 1,
          "MinHealthyTime": 0,
          "ProgressDeadline": 0,
          "Stagger": 30000000000
        },
        "VaultNamespace": "",
        "VaultToken": "",
        "Version": 0
      }
    ]
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._nomad import argument_spec as nomad_argument_spec
from ansible_collections.community.general.plugins.module_utils._nomad import setup_nomad_client


def run():
    module = AnsibleModule(
        argument_spec={
            **nomad_argument_spec,
            "name": dict(type="str"),
        },
        supports_check_mode=True,
    )

    nomad_client = setup_nomad_client(module)

    changed = False
    result = list()
    try:
        job_list = nomad_client.jobs.get_jobs()
        for job in job_list:
            result.append(nomad_client.job.get_job(job.get("ID")))
    except Exception as e:
        module.fail_json(msg=f"{e}")

    if module.params["name"]:
        filter = list()
        try:
            for job in result:
                if job.get("ID") == module.params["name"]:
                    filter.append(job)
                    result = filter
            if not filter:
                module.fail_json(msg=f"Couldn't find Job with id {module.params['name']}")
        except Exception as e:
            module.fail_json(msg=f"{e}")

    module.exit_json(changed=changed, result=result)


def main():
    run()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2026, Tom Scholz <tomscholz@outlook.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: google_chat
short_description: Send Google Chat notifications
version_added: "13.1.0"
description:
  - Sends notifications to a Google Chat space using an incoming webhook.
  - Incoming webhooks are one-way. They send messages but cannot receive or respond to them.
author:
  - Tom Scholz (@tomscholz)
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  space:
    type: str
    required: true
    description:
      - The identifier of the Chat space to post to, taken from the incoming webhook URL.
      - For a webhook URL of the form C(https://chat.googleapis.com/v1/spaces/AAAA/messages?key=...&token=...),
        this is the C(AAAA) part.
  key:
    type: str
    required: true
    description:
      - The C(key) request parameter from the incoming webhook URL.
      - Keep this value secret as it grants the ability to post to the space.
  token:
    type: str
    required: true
    description:
      - The C(token) request parameter from the incoming webhook URL.
      - Keep this value secret as it grants the ability to post to the space.
  text:
    type: str
    required: true
    description:
      - The text of the message to send.
      - 'Emoji must be supplied as Unicode characters (for example V(🚀)). The Chat API does not
        render C(:shortcode:) style emoji in plain text messages as they appear as literal text.'
  thread_key:
    type: str
    description:
      - An arbitrary key used to start or reply to a message thread.
      - When set, O(create_new_thread) controls the behavior when the thread is not found.
  create_new_thread:
    type: bool
    default: true
    description:
      - Controls behavior when O(thread_key) is set but no matching thread exists.
      - When V(true), a new thread is started if no matching thread is found.
      - When V(false), the message is only posted if a matching thread already exists, otherwise it fails.
      - Only used when O(thread_key) is set.
seealso:
  - name: Google Chat incoming webhooks
    description: Google's reference for sending messages to Chat with incoming webhooks.
    link: https://developers.google.com/workspace/chat/quickstart/webhooks
"""

EXAMPLES = r"""
- name: Send a notification to Google Chat
  community.general.google_chat:
    space: SPACE_ID
    key: KEY
    token: TOKEN
    text: '{{ inventory_hostname }} completed'
  delegate_to: localhost

- name: Start a thread
  community.general.google_chat:
    space: SPACE_ID
    key: KEY
    token: TOKEN
    text: 'Starting a thread'
    thread_key: 'deploy-2026-06-01'
    create_new_thread: true

# Post each deploy step into a single thread. The first message creates the thread
# with create_new_thread=true. Follow-ups use create_new_thread=false so they only
# post if the opening message went through, rather than leaving orphan threads.
# Note: webhooks are rate-limited to 1 request per second per space.
- name: Announce deploy start (starts the thread)
  community.general.google_chat:
    space: "{{ chat_space }}"
    key: "{{ chat_key }}"
    token: "{{ chat_token }}"
    text: "🚀 Starting deploy of *{{ app_version | default('latest') }}* to {{ inventory_hostname }}"
    thread_key: "{{ deploy_thread }}"
    create_new_thread: true
  delegate_to: localhost
  run_once: true
  # deploy_thread is defined once for the play, for example:
  #   deploy_thread: "deploy-{{ inventory_hostname }}-{{ ansible_date_time.iso8601_basic_short }}"

- name: Report a step into the same thread
  community.general.google_chat:
    space: "{{ chat_space }}"
    key: "{{ chat_key }}"
    token: "{{ chat_token }}"
    text: "✅ Step 1/3 – code checked out"
    thread_key: "{{ deploy_thread }}"
    create_new_thread: false
  delegate_to: localhost
  run_once: true

# Wrap risky tasks so a failure posts to the same thread before a play aborts.
- name: Deploy with failure notification
  block:
    - name: Restart service
      ansible.builtin.systemd:
        name: app
        state: restarted

    - name: Report success
      community.general.google_chat:
        space: "{{ chat_space }}"
        key: "{{ chat_key }}"
        token: "{{ chat_token }}"
        text: "🎉 Deploy to {{ inventory_hostname }} complete"
        thread_key: "{{ deploy_thread }}"
        create_new_thread: false
      delegate_to: localhost
      run_once: true
  rescue:
    - name: Report failure into the thread
      community.general.google_chat:
        space: "{{ chat_space }}"
        key: "{{ chat_key }}"
        token: "{{ chat_token }}"
        text: "❌ Deploy to {{ inventory_hostname }} *failed* – {{ ansible_failed_task.name }}"
        thread_key: "{{ deploy_thread }}"
        create_new_thread: false
      delegate_to: localhost
      run_once: true

    - name: Re-raise the failure
      ansible.builtin.fail:
        msg: "Deploy failed at {{ ansible_failed_task.name }}"
"""

RETURN = r"""
name:
  description: Resource name of the created message, returned by the Chat API.
  returned: success
  type: str
  sample: "spaces/AAAA/messages/BBBB.BBBB"
thread_name:
  description: Resource name of the thread the message belongs to.
  returned: when the response includes a thread
  type: str
  sample: "spaces/AAAA/threads/CCCC"
"""

import typing as t
from urllib.parse import urlencode

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url

BASE_URL = "https://chat.googleapis.com/v1/spaces"

if t.TYPE_CHECKING:
    Payload = dict[str, t.Any]
    Response = dict[str, t.Any]


def build_payload(text: str, thread_key: str | None) -> Payload:
    payload: Payload = {"text": text}

    if thread_key is not None:
        payload["thread"] = {"threadKey": thread_key}
    return payload


def build_url(space: str, key: str, token: str, thread_key: str | None, create_new_thread: bool) -> str:
    params = {"key": key, "token": token}
    if thread_key is not None:
        params["messageReplyOption"] = (
            "REPLY_MESSAGE_FALLBACK_TO_NEW_THREAD" if create_new_thread else "REPLY_MESSAGE_OR_FAIL"
        )
    return f"{BASE_URL}/{space}/messages?{urlencode(params)}"


def do_notify(module: AnsibleModule, url: str, payload: Payload) -> Response:
    headers = {
        "Content-Type": "application/json; charset=UTF-8",
        "Accept": "application/json",
    }
    data = module.jsonify(payload)
    response, info = fetch_url(module=module, url=url, headers=headers, method="POST", data=data)

    if info["status"] != 200:
        body = info.get("body")
        if hasattr(body, "decode"):
            body = body.decode("utf-8", errors="replace")
        module.fail_json(
            msg=f"Failed to send message to Google Chat (HTTP {info['status']}): {body or info.get('msg')}"
        )

    return module.from_json(response.read())


def main() -> None:
    module = AnsibleModule(
        argument_spec=dict(
            space=dict(type="str", required=True),
            key=dict(type="str", required=True, no_log=True),
            token=dict(type="str", required=True, no_log=True),
            text=dict(type="str", required=True),
            thread_key=dict(type="str", no_log=False),
            create_new_thread=dict(type="bool", default=True),
        ),
        supports_check_mode=True,
    )

    if module.check_mode:
        module.exit_json(changed=True)

    payload = build_payload(module.params["text"], module.params["thread_key"])
    url = build_url(
        module.params["space"],
        module.params["key"],
        module.params["token"],
        module.params["thread_key"],
        module.params["create_new_thread"],
    )

    response = do_notify(module, url, payload)

    result = {"changed": True}
    if "name" in response:
        result["name"] = response["name"]
    if isinstance(response.get("thread"), dict) and "name" in response["thread"]:
        result["thread_name"] = response["thread"]["name"]

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2019, Guillaume Martinez (lunik@tiwabbit.fr)
# Copyright (c) 2018, Marcus Watkins <marwatk@marcuswatkins.net>
# Based on code:
# Copyright (c) 2013, Phillip Gentry <phillip@cx.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: gitlab_deploy_key
short_description: Manages GitLab project deploy keys
description:
  - Adds, updates and removes project deploy keys.
author:
  - Marcus Watkins (@marwatk)
  - Guillaume Martinez (@Lunik)
requirements:
  - python-gitlab python module
extends_documentation_fragment:
  - community.general._auth_basic
  - community.general._gitlab
  - community.general._attributes

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none

options:
  project:
    description:
      - ID or Full path of project in the form of group/name.
    required: true
    type: str
  title:
    description:
      - Deploy key's title.
    required: true
    type: str
  key:
    description:
      - Deploy key.
    required: true
    type: str
  can_push:
    description:
      - Whether this key can push to the project.
    type: bool
    default: false
  state:
    description:
      - When V(present) the deploy key is added to the project if it does not exist.
      - When V(absent) it is removed from the project if it exists.
    default: present
    type: str
    choices: ["present", "absent"]
"""

EXAMPLES = r"""
- name: "Adding a project deploy key"
  community.general.gitlab_deploy_key:
    api_url: https://gitlab.example.com/
    api_token: "{{ api_token }}"
    project: "my_group/my_project"
    title: "Jenkins CI"
    state: present
    key: "ssh-rsa AAAAB3NzaC1yc2EAAAABJQAAAIEAiPWx6WM4lhHNedGfBpPJNPpZ7yKu+dnn1SJejgt4596k6YjzGGphH2TUxwKzxcKDKKezwkpfnxPkSMkuEspGRt/aZZ9w..."

- name: "Update the above deploy key to add push access"
  community.general.gitlab_deploy_key:
    api_url: https://gitlab.example.com/
    api_token: "{{ api_token }}"
    project: "my_group/my_project"
    title: "Jenkins CI"
    state: present
    can_push: true

- name: "Remove the previous deploy key from the project"
  community.general.gitlab_deploy_key:
    api_url: https://gitlab.example.com/
    api_token: "{{ api_token }}"
    project: "my_group/my_project"
    state: absent
    key: "ssh-rsa AAAAB3NzaC1yc2EAAAABJQAAAIEAiPWx6WM4lhHNedGfBpPJNPpZ7yKu+dnn1SJejgt4596k6YjzGGphH2TUxwKzxcKDKKezwkpfnxPkSMkuEspGRt/aZZ9w..."
"""

RETURN = r"""
msg:
  description: Success or failure message.
  returned: always
  type: str
  sample: "Success"

result:
  description: JSON-parsed response from the server.
  returned: always
  type: dict

error:
  description: The error message returned by the GitLab API.
  returned: failed
  type: str
  sample: "400: key is already in use"

deploy_key:
  description: API object.
  returned: always
  type: dict
"""

from ansible.module_utils.api import basic_auth_argument_spec
from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._gitlab import (
    auth_argument_spec,
    find_project,
    gitlab,
    gitlab_authentication,
    list_all_kwargs,
)


class GitLabDeployKey:
    def __init__(self, module, gitlab_instance):
        self._module = module
        self._gitlab = gitlab_instance
        self.deploy_key_object = None

    """
    @param project Project object
    @param key_title Title of the key
    @param key_key String of the key
    @param key_can_push Option of the deploy_key
    @param options Deploy key options
    """

    def create_or_update_deploy_key(self, project, key_title, key_key, options):
        changed = False

        # note: unfortunately public key cannot be updated directly by
        #   GitLab REST API, so for that case we need to delete and
        #   than recreate the key
        if self.deploy_key_object and self.deploy_key_object.key != key_key:
            if not self._module.check_mode:
                self.deploy_key_object.delete()
            self.deploy_key_object = None

        # Because we have already call exists_deploy_key in main()
        if self.deploy_key_object is None:
            deploy_key = self.create_deploy_key(
                project, {"title": key_title, "key": key_key, "can_push": options["can_push"]}
            )
            changed = True
        else:
            changed, deploy_key = self.update_deploy_key(
                self.deploy_key_object, {"title": key_title, "can_push": options["can_push"]}
            )

        self.deploy_key_object = deploy_key
        if changed:
            if self._module.check_mode:
                self._module.exit_json(changed=True, msg=f"Successfully created or updated the deploy key {key_title}")

            try:
                deploy_key.save()
            except Exception as e:
                self._module.fail_json(msg=f"Failed to update deploy key: {e} ")
            return True
        else:
            return False

    """
    @param project Project Object
    @param arguments Attributes of the deploy_key
    """

    def create_deploy_key(self, project, arguments):
        if self._module.check_mode:
            return True

        try:
            deploy_key = project.keys.create(arguments)
        except gitlab.exceptions.GitlabCreateError as e:
            self._module.fail_json(msg=f"Failed to create deploy key: {e} ")

        return deploy_key

    """
    @param deploy_key Deploy Key Object
    @param arguments Attributes of the deploy_key
    """

    def update_deploy_key(self, deploy_key, arguments):
        changed = False

        for arg_key, arg_value in arguments.items():
            if arg_value is not None:
                if getattr(deploy_key, arg_key) != arg_value:
                    setattr(deploy_key, arg_key, arg_value)
                    changed = True

        return (changed, deploy_key)

    """
    @param project Project object
    @param key_title Title of the key
    """

    def find_deploy_key(self, project, key_title):
        for deploy_key in project.keys.list(**list_all_kwargs):
            if deploy_key.title == key_title:
                return deploy_key

    """
    @param project Project object
    @param key_title Title of the key
    """

    def exists_deploy_key(self, project, key_title):
        # When project exists, object will be stored in self.project_object.
        deploy_key = self.find_deploy_key(project, key_title)
        if deploy_key:
            self.deploy_key_object = deploy_key
            return True
        return False

    def delete_deploy_key(self):
        if self._module.check_mode:
            return True

        return self.deploy_key_object.delete()


def main():
    argument_spec = basic_auth_argument_spec()
    argument_spec.update(auth_argument_spec())
    argument_spec.update(
        dict(
            state=dict(type="str", default="present", choices=["absent", "present"]),
            project=dict(type="str", required=True),
            key=dict(type="str", required=True, no_log=False),
            can_push=dict(type="bool", default=False),
            title=dict(type="str", required=True),
        )
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=[
            ["api_username", "api_token"],
            ["api_username", "api_oauth_token"],
            ["api_username", "api_job_token"],
            ["api_token", "api_oauth_token"],
            ["api_token", "api_job_token"],
        ],
        required_together=[["api_username", "api_password"]],
        required_one_of=[["api_username", "api_token", "api_oauth_token", "api_job_token"]],
        supports_check_mode=True,
    )

    # check prerequisites and connect to gitlab server
    gitlab_instance = gitlab_authentication(module)

    state = module.params["state"]
    project_identifier = module.params["project"]
    key_title = module.params["title"]
    key_keyfile = module.params["key"]
    key_can_push = module.params["can_push"]

    gitlab_deploy_key = GitLabDeployKey(module, gitlab_instance)

    project = find_project(gitlab_instance, project_identifier)

    if project is None:
        module.fail_json(msg=f"Failed to create deploy key: project {project_identifier} doesn't exists")

    deploy_key_exists = gitlab_deploy_key.exists_deploy_key(project, key_title)

    if state == "absent":
        if deploy_key_exists:
            gitlab_deploy_key.delete_deploy_key()
            module.exit_json(changed=True, msg=f"Successfully deleted deploy key {key_title}")
        else:
            module.exit_json(changed=False, msg="Deploy key deleted or does not exists")

    if state == "present":
        if gitlab_deploy_key.create_or_update_deploy_key(project, key_title, key_keyfile, {"can_push": key_can_push}):
            module.exit_json(
                changed=True,
                msg=f"Successfully created or updated the deploy key {key_title}",
                deploy_key=gitlab_deploy_key.deploy_key_object._attrs,
            )
        else:
            module.exit_json(
                changed=False,
                msg=f"No need to update the deploy key {key_title}",
                deploy_key=gitlab_deploy_key.deploy_key_object._attrs,
            )


if __name__ == "__main__":
    main()



#
# (c) 2020 Red Hat Inc.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
    short_description: RAM backed, non persistent cache.
    description:
        - RAM backed cache that is not persistent.
        - Tailored for networking use case.
    version_added: 2.0.0
    author:
        - Ansible Networking Team (@ansible-network)
    name: memory
"""

from ansible.plugins import AnsiblePlugin


class CacheModule(AnsiblePlugin):
    _persistent = False

    def __init__(self, *args, **kwargs):
        super(CacheModule, self).__init__(*args, **kwargs)
        self._cache = {}

    def get(self, key):
        return self._cache.get(key)

    def set(self, key, value):
        self._cache[key] = value

    def keys(self):
        return self._cache.keys()

    def flush(self):
        self._cache = {}

    def lookup(self, key):
        return self.get(key)

    def populate(self, key, value):
        self.set(key, value)

    def invalidate(self):
        self.flush()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: eks_cluster
version_added: 1.0.0
short_description: Manage Elastic Kubernetes Service (EKS) Clusters
description:
  - Manage Elastic Kubernetes Service (EKS) Clusters.
  - Prior to release 5.0.0 this module was called C(community.aws.aws_eks_cluster).
    The usage did not change.
author:
  - Will Thames (@willthames)
options:
  name:
    description: Name of the EKS cluster.
    required: True
    type: str
  version:
    description:
      - Kubernetes version.
      - Defaults to C(latest).
    type: str
  role_arn:
    description: ARN of IAM role used by the EKS cluster.
    type: str
  subnets:
    description: List of subnet IDs for the Kubernetes cluster.
    type: list
    elements: str
  security_groups:
    description: List of security group names or IDs.
    type: list
    elements: str
  state:
    description: Desired state of the EKS cluster.
    choices:
      - absent
      - present
    default: present
    type: str
  tags:
    description:
      - A dictionary of tags to add the EKS cluster.
    type: dict
    version_added: 5.3.0
  wait:
    description: >-
      Specifies whether the module waits until the cluster is active or deleted
      before moving on. It takes "usually less than 10 minutes" per AWS documentation.
    type: bool
    default: false
  wait_timeout:
    description: >-
      The duration in seconds to wait for the cluster to become active. Defaults
      to 1200 seconds (20 minutes).
    default: 1200
    type: int
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Create an EKS cluster
  community.aws.eks_cluster:
    name: my_cluster
    version: 1.14
    role_arn: my_eks_role
    subnets:
      - subnet-aaaa1111
    security_groups:
      - my_eks_sg
      - sg-abcd1234
  register: caller_facts

- name: Remove an EKS cluster
  community.aws.eks_cluster:
    name: my_cluster
    wait: true
    state: absent
"""

RETURN = r"""
arn:
  description: ARN of the EKS cluster
  returned: when state is present
  type: str
  sample: arn:aws:eks:us-west-2:123456789012:cluster/my-eks-cluster
certificate_authority:
  description: Dictionary containing Certificate Authority Data for cluster
  returned: after creation
  type: complex
  contains:
    data:
      description: Base-64 encoded Certificate Authority Data for cluster
      returned: when the cluster has been created and is active
      type: str
endpoint:
  description: Kubernetes API server endpoint
  returned: when the cluster has been created and is active
  type: str
  sample: https://API_SERVER_ENDPOINT.yl4.us-west-2.eks.amazonaws.com
created_at:
  description: Cluster creation date and time
  returned: when state is present
  type: str
  sample: '2018-06-06T11:56:56.242000+00:00'
name:
  description: EKS cluster name
  returned: when state is present
  type: str
  sample: my-eks-cluster
resources_vpc_config:
  description: VPC configuration of the cluster
  returned: when state is present
  type: complex
  contains:
    security_group_ids:
      description: List of security group IDs
      returned: always
      type: list
      sample:
      - sg-abcd1234
      - sg-aaaa1111
    subnet_ids:
      description: List of subnet IDs
      returned: always
      type: list
      sample:
      - subnet-abcdef12
      - subnet-345678ab
      - subnet-cdef1234
    vpc_id:
      description: VPC id
      returned: always
      type: str
      sample: vpc-a1b2c3d4
role_arn:
  description: ARN of the IAM role used by the cluster
  returned: when state is present
  type: str
  sample: arn:aws:iam::123456789012:role/eks_cluster_role
status:
  description: status of the EKS cluster
  returned: when state is present
  type: str
  sample:
  - CREATING
  - ACTIVE
version:
  description: Kubernetes version of the cluster
  returned: when state is present
  type: str
  sample: '1.10'
"""

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import get_ec2_security_group_ids_from_names
from ansible_collections.amazon.aws.plugins.module_utils.waiters import get_waiter

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def ensure_present(client, module):
    name = module.params.get("name")
    subnets = module.params["subnets"]
    groups = module.params["security_groups"]
    wait = module.params.get("wait")
    cluster = get_cluster(client, module)
    try:
        ec2 = module.client("ec2")
        vpc_id = ec2.describe_subnets(SubnetIds=[subnets[0]])["Subnets"][0]["VpcId"]
        groups = get_ec2_security_group_ids_from_names(groups, ec2, vpc_id)
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(e, msg="Couldn't lookup security groups")

    if cluster:
        if set(cluster["resourcesVpcConfig"]["subnetIds"]) != set(subnets):
            module.fail_json(msg="Cannot modify subnets of existing cluster")
        if set(cluster["resourcesVpcConfig"]["securityGroupIds"]) != set(groups):
            module.fail_json(msg="Cannot modify security groups of existing cluster")
        if module.params.get("version") and module.params.get("version") != cluster["version"]:
            module.fail_json(msg="Cannot modify version of existing cluster")

        if wait:
            wait_until(client, module, "cluster_active")
            # Ensure that fields that are only available for active clusters are
            # included in the returned value
            cluster = get_cluster(client, module)

        module.exit_json(changed=False, **camel_dict_to_snake_dict(cluster))

    if module.check_mode:
        module.exit_json(changed=True)
    try:
        params = dict(
            name=name,
            roleArn=module.params["role_arn"],
            resourcesVpcConfig=dict(subnetIds=subnets, securityGroupIds=groups),
        )
        if module.params["version"]:
            params["version"] = module.params["version"]
        if module.params["tags"]:
            params["tags"] = module.params["tags"]
        cluster = client.create_cluster(**params)["cluster"]
    except botocore.exceptions.EndpointConnectionError:
        module.fail_json(msg=f"Region {client.meta.region_name} is not supported by EKS")
    except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
        module.fail_json_aws(e, msg=f"Couldn't create cluster {name}")

    if wait:
        wait_until(client, module, "cluster_active")
        # Ensure that fields that are only available for active clusters are
        # included in the returned value
        cluster = get_cluster(client, module)

    module.exit_json(changed=True, **camel_dict_to_snake_dict(cluster))


def ensure_absent(client, module):
    name = module.params.get("name")
    existing = get_cluster(client, module)
    wait = module.params.get("wait")
    if not existing:
        module.exit_json(changed=False)
    if not module.check_mode:
        try:
            client.delete_cluster(name=module.params["name"])
        except botocore.exceptions.EndpointConnectionError:
            module.fail_json(msg=f"Region {client.meta.region_name} is not supported by EKS")
        except (botocore.exceptions.BotoCoreError, botocore.exceptions.ClientError) as e:
            module.fail_json_aws(e, msg=f"Couldn't delete cluster {name}")

    if wait:
        wait_until(client, module, "cluster_deleted")

    module.exit_json(changed=True)


def get_cluster(client, module):
    name = module.params.get("name")
    try:
        return client.describe_cluster(name=name)["cluster"]
    except is_boto3_error_code("ResourceNotFoundException"):
        return None
    except botocore.exceptions.EndpointConnectionError:  # pylint: disable=duplicate-except
        module.fail_json(msg=f"Region {client.meta.region_name} is not supported by EKS")
    except (
        botocore.exceptions.BotoCoreError,
        botocore.exceptions.ClientError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg=f"Couldn't get cluster {name}")


def wait_until(client, module, waiter_name="cluster_active"):
    name = module.params.get("name")
    wait_timeout = module.params.get("wait_timeout")

    waiter = get_waiter(client, waiter_name)
    attempts = 1 + int(wait_timeout / waiter.config.delay)
    waiter.wait(name=name, WaiterConfig={"MaxAttempts": attempts})


def main():
    argument_spec = dict(
        name=dict(required=True),
        version=dict(),
        role_arn=dict(),
        subnets=dict(type="list", elements="str"),
        security_groups=dict(type="list", elements="str"),
        state=dict(choices=["absent", "present"], default="present"),
        tags=dict(type="dict", required=False),
        wait=dict(default=False, type="bool"),
        wait_timeout=dict(default=1200, type="int"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        required_if=[["state", "present", ["role_arn", "subnets", "security_groups"]]],
        supports_check_mode=True,
    )

    client = module.client("eks")

    if module.params.get("state") == "present":
        ensure_present(client, module)
    else:
        ensure_absent(client, module)


if __name__ == "__main__":
    main()



# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import functools
import typing as t

from .. import errors
from . import utils

if t.TYPE_CHECKING:
    from collections.abc import Callable

    from ..api.client import APIClient

    _Self = t.TypeVar("_Self")
    _P = t.ParamSpec("_P")
    _R = t.TypeVar("_R")


def minimum_version(
    version: str,
) -> Callable[
    [Callable[t.Concatenate[_Self, _P], _R]],
    Callable[t.Concatenate[_Self, _P], _R],
]:
    def decorator(
        f: Callable[t.Concatenate[_Self, _P], _R],
    ) -> Callable[t.Concatenate[_Self, _P], _R]:
        @functools.wraps(f)
        def wrapper(self: _Self, *args: _P.args, **kwargs: _P.kwargs) -> _R:
            # We use _Self instead of APIClient since this is used for mixins for APIClient.
            # This unfortunately means that self._version does not exist in the mixin,
            # it only exists after mixing in. This is why we ignore types here.
            if utils.version_lt(self._version, version):  # type: ignore
                raise errors.InvalidVersion(
                    f"{f.__name__} is not available for version < {version}"
                )
            return f(self, *args, **kwargs)

        return wrapper

    return decorator


def update_headers(
    f: Callable[t.Concatenate[APIClient, _P], _R],
) -> Callable[t.Concatenate[APIClient, _P], _R]:
    def inner(self: APIClient, *args: _P.args, **kwargs: _P.kwargs) -> _R:
        if "HttpHeaders" in self._general_configs:
            if not kwargs.get("headers"):
                kwargs["headers"] = self._general_configs["HttpHeaders"]
            else:
                # We cannot (yet) model that kwargs["headers"] should be a dictionary
                kwargs["headers"].update(self._general_configs["HttpHeaders"])  # type: ignore
        return f(self, *args, **kwargs)

    return inner



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_workspace_info
version_added: "3.15.0"
short_description: List ML Workspaces
description:
    - List ML Workspaces.
options:
    name:
        description:
            - Retrieve details about a specific ml workspace.
        type: str
        required: false
    filtered_kinds:
        description:
            - List only the specified kinds of ml workspaces.
        required: false
        type: list
        elements: str
        default: []
        choices:
            - default
            - hub
            - project
    resource_group:
        description:
            - Name of a resource group to limit ml workspaces from.
              If not specified the subscription will be searched.
        required: false
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Get info on a specific ml workspaces
  azure.azcollection.azure_rm_ml_workspace_info:
    name: my_workspace
    resource_group: test_ml_workspace
  register: result

- name: List all ml workspaces in subscription
  azure.azcollection.azure_rm_ml_workspace_info:
  register: results

- name: List all hub workspaces in resource group
  azure.azcollection.azure_rm_ml_workspace_info:
    resource_group: test_ml_workspace
    filtered_kinds:
      - hub
  register: results
'''

RETURN = '''
ml_workspaces:
    description:
        - List of ML Workspaces.
    returned: always
    type: dict
    sample: [
        {
            "allow_roleassignment_on_rg": true,
            "application_insights": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/test-ml-workspace/providers/Microsoft.insights/components/workspacinsightsxxxxxxxx",
            "description": "Workspace Integration Tests",
            "discovery_url": "https://westus3.api.azureml.ms/discovery",
            "display_name": "workspace-xxxxxxx-xxx",
            "enable_data_isolation": false,
            "hbi_workspace": false,
            "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/test-ml-workspace/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxx-xxx",
            "identity": {
                "principal_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "tenant_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "type": "system_assigned"
            },
            "key_vault": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/test-ml-workspace/providers/Microsoft.Keyvault/vaults/workspackeyvaultxxxxxxxx",
            "location": "westus3",
            "managed_network": {
                "firewall_sku": "standard",
                "isolation_mode": "allow_only_approved_outbound",
                "network_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "outbound_rules": [
                    {
                        "category": "user_defined",
                        "destination": "microsoft.com",
                        "name": "microsoft",
                        "status": "Inactive",
                        "type": "fqdn"
                    },
                    {
                        "category": "user_defined",
                        "destination": {
                            "address_prefixes": [
                                "168.63.129.16",
                                "10.0.0.0/24"
                            ],
                            "port_ranges": "80, 8080-8089",
                            "protocol": "TCP",
                            "service_tag": "sometag"
                        },
                        "name": "servicetag-w-prefixes",
                        "status": "Inactive",
                        "type": "service_tag"
                    },
                    {
                        "category": "required",
                        "destination": {
                            "address_prefixes": [],
                            "port_ranges": "443",
                            "protocol": "TCP",
                            "service_tag": "AzureMonitor"
                        },
                        "name": "__SYS_ST_AzureMonitor_TCP",
                        "status": "Inactive",
                        "type": "service_tag"
                    },
                    {
                        "category": "required",
                        "destination": {
                            "address_prefixes": [],
                            "port_ranges": "443",
                            "protocol": "TCP",
                            "service_tag": "AzureResourceManager"
                        },
                        "name": "__SYS_ST_AzureResourceManager_TCP",
                        "status": "Inactive",
                        "type": "service_tag"
                    },
                    {
                        "category": "required",
                        "destination": {
                            "service_resource_id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/test-ml-workspace/providers/Microsoft.Storage/storageAccounts/workspacstoragexxxxxxxxx",
                            "spark_enabled": true,
                            "subresource_target": "file"
                        },
                        "name": "__SYS_PE_workspacstoragexxxxxxxxx_file_xxxxxxxx",
                        "status": "Inactive",
                        "type": "private_endpoint"
                    },
                    {
                        "category": "required",
                        "destination": {
                            "address_prefixes": [],
                            "port_ranges": "443",
                            "protocol": "TCP",
                            "service_tag": "MicrosoftContainerRegistry.westus3"
                        },
                        "name": "__SYS_ST_MicrosoftContainerRegistry.westus3_TCP",
                        "status": "Inactive",
                        "type": "service_tag"
                    },
                    {
                        "category": "required",
                        "destination": {
                            "address_prefixes": [],
                            "port_ranges": "*",
                            "protocol": "*",
                            "service_tag": "VirtualNetwork"
                        },
                        "name": "__SYS_ST_VirtualNetwork",
                        "status": "Inactive",
                        "type": "service_tag"
                    },
                    {
                        "category": "required",
                        "destination": {
                            "address_prefixes": [],
                            "port_ranges": "443",
                            "protocol": "TCP",
                            "service_tag": "BatchNodeManagement"
                        },
                        "name": "__SYS_ST_BatchNodeManagement_TCP",
                        "status": "Inactive",
                        "type": "service_tag"
                    },
                    {
                        "category": "required",
                        "destination": {
                            "address_prefixes": [],
                            "port_ranges": "80, 443",
                            "protocol": "TCP",
                            "service_tag": "AzureActiveDirectory"
                        },
                        "name": "__SYS_ST_AzureActiveDirectory_TCP",
                        "status": "Inactive",
                        "type": "service_tag"
                    },
                    {
                        "category": "required",
                        "destination": {
                            "service_resource_id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/test-ml-workspace/providers/Microsoft.Storage/storageAccounts/workspacstoragexxxxxxxxx",
                            "spark_enabled": true,
                            "subresource_target": "blob"
                        },
                        "name": "__SYS_PE_workspacstoragexxxxxxxxx_blob_xxxxxxxx",
                        "status": "Inactive",
                        "type": "private_endpoint"
                    },
                    {
                        "category": "required",
                        "destination": {
                            "service_resource_id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/test-ml-workspace/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxx-xxx",
                            "spark_enabled": true,
                            "subresource_target": "amlworkspace"
                        },
                        "name": "__SYS_PE_workspace-xxxxxxx-xxx_amlworkspace_xxxxxxxx",
                        "status": "Inactive",
                        "type": "private_endpoint"
                    },
                    {
                        "category": "required",
                        "destination": {
                            "address_prefixes": [],
                            "port_ranges": "*",
                            "protocol": "*",
                            "service_tag": "AzureFrontDoor.FirstParty"
                        },
                        "name": "__SYS_ST_AzureFrontDoor.FirstParty",
                        "status": "Inactive",
                        "type": "service_tag"
                    },
                    {
                        "category": "required",
                        "destination": {
                            "address_prefixes": [],
                            "port_ranges": "443",
                            "protocol": "TCP",
                            "service_tag": "AzureMachineLearning"
                        },
                        "name": "__SYS_ST_AzureMachineLearning_TCP",
                        "status": "Inactive",
                        "type": "service_tag"
                    },
                    {
                        "category": "required",
                        "destination": {
                            "service_resource_id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/test-ml-workspace/providers/Microsoft.Keyvault/vaults/workspackeyvaultxxxxxxxx",
                            "spark_enabled": true,
                            "subresource_target": "vault"
                        },
                        "name": "__SYS_PE_workspackeyvault2aa0c6bf_vault_xxxxxxxx",
                        "status": "Inactive",
                        "type": "private_endpoint"
                    },
                    {
                        "category": "required",
                        "destination": {
                            "address_prefixes": [],
                            "port_ranges": "5831",
                            "protocol": "UDP",
                            "service_tag": "AzureMachineLearning"
                        },
                        "name": "__SYS_ST_AzureMachineLearning_UDP",
                        "status": "Inactive",
                        "type": "service_tag"
                    }
                ],
                "status": {
                    "spark_ready": false,
                    "status": "Inactive"
                }
            },
            "mlflow_tracking_uri": "azureml://westus3.api.azureml.ms/mlflow/v1.0/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/test-ml-workspace/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxx-xxx",
            "name": "workspace-xxxxxxx-xxx",
            "network_acls": {
                "default_action": "Allow",
                "ip_rules": []
            },
            "provision_network_now": false,
            "public_network_access": "Disabled",
            "resource_group": "test-ml-workspace",
            "serverless_compute": {
                "no_public_ip": false
            },
            "storage_account": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/test-ml-workspace/providers/Microsoft.Storage/storageAccounts/workspacstoragefxxxxxxxx",
            "system_datastores_auth_mode": "accesskey",
            "tags": {}
        }
    ]
'''  # NOQA


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
    from azure.ai.ml.constants._common import Scope
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLWorkspaceInfo(MLClientCommon):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=False
            ),
            filtered_kinds=dict(
                type='list',
                required=False,
                default=[],
                elements='str',
                choices=['default',
                         'hub',
                         'project']
            ),
            resource_group=dict(
                type='str',
                required=False
            ),
        )

        self._client = None
        self.ml_workspace = None
        self.ml_registry = None

        required_by = {'name': 'resource_group'}

        self.results = dict(
            ml_workspaces=[]
        )

        super(AzureRMMLWorkspaceInfo, self).__init__(self.module_arg_spec,
                                                     supports_tags=False,
                                                     supports_check_mode=True,
                                                     required_by=required_by,
                                                     facts_module=True,
                                                     )

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name:
            try:
                result = self.client.workspaces.get(self.name)
                ml_workspaces = [self.entity_to_dict(result)]
            except ResourceNotFoundError:
                ml_workspaces = []
        else:
            scope = Scope.RESOURCE_GROUP if self.resource_group else Scope.SUBSCRIPTION
            results = self.client.workspaces.list(scope=scope,
                                                  filtered_kinds=','.join(self.filtered_kinds))
            ml_workspaces = list(map(self.entity_to_dict, results))

        self.results['ml_workspaces'] = ml_workspaces

        return self.results


def main():
    AzureRMMLWorkspaceInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_switch_routing_multicast_rendezvous_points
short_description: Resource module for networks _switch _routing _multicast _rendezvous
  _points
description:
  - Manage operations create, update and delete of the resource networks _switch _routing
    _multicast _rendezvous _points.
  - Create a multicast rendezvous point.
  - Delete a multicast rendezvous point.
  - Update a multicast rendezvous point.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  interfaceIp:
    description: The IP address of the interface where the RP needs to be created.
    type: str
  multicastGroup:
    description: '''Any'', or the IP address of a multicast group.'
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  rendezvousPointId:
    description: RendezvousPointId path parameter. Rendezvous point ID.
    type: str
  vrf:
    description: The VRF with PIM enabled L3 interface.
    suboptions:
      name:
        description: The name of the VRF.
        type: str
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for switch createNetworkSwitchRoutingMulticastRendezvousPoint
    description: Complete reference of the createNetworkSwitchRoutingMulticastRendezvousPoint
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-switch-routing-multicast-rendezvous-point
  - name: Cisco Meraki documentation for switch deleteNetworkSwitchRoutingMulticastRendezvousPoint
    description: Complete reference of the deleteNetworkSwitchRoutingMulticastRendezvousPoint
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-network-switch-routing-multicast-rendezvous-point
  - name: Cisco Meraki documentation for switch updateNetworkSwitchRoutingMulticastRendezvousPoint
    description: Complete reference of the updateNetworkSwitchRoutingMulticastRendezvousPoint
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-routing-multicast-rendezvous-point
notes:
  - SDK Method used are
    switch.Switch.create_network_switch_routing_multicast_rendezvous_point,
    switch.Switch.delete_network_switch_routing_multicast_rendezvous_point,
    switch.Switch.update_network_switch_routing_multicast_rendezvous_point,
  - Paths used are
    post /networks/{networkId}/switch/routing/multicast/rendezvousPoints,
    delete /networks/{networkId}/switch/routing/multicast/rendezvousPoints/{rendezvousPointId},
    put /networks/{networkId}/switch/routing/multicast/rendezvousPoints/{rendezvousPointId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_switch_routing_multicast_rendezvous_points:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    interfaceIp: 192.168.1.2
    multicastGroup: Any
    networkId: string
    vrf:
      name: Blue
- name: Delete by id
  cisco.meraki.networks_switch_routing_multicast_rendezvous_points:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    networkId: string
    rendezvousPointId: string
- name: Update by id
  cisco.meraki.networks_switch_routing_multicast_rendezvous_points:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    interfaceIp: 192.168.1.2
    multicastGroup: Any
    networkId: string
    rendezvousPointId: string
    vrf:
      name: Blue
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "rendezvousPointId": "string",
      "serial": "string",
      "interfaceName": "string",
      "interfaceIp": "string",
      "multicastGroup": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_wireless_radio_rrm
short_description: Resource module for networks _wireless _radio _rrm
description:
  - Manage operation update of the resource networks _wireless _radio _rrm.
  - Update the AutoRF settings for a wireless network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  ai:
    description: AI settings.
    suboptions:
      enabled:
        description: Toggle for enabling or disabling AI in a network.
        type: bool
    type: dict
  busyHour:
    description: Busy Hour settings.
    suboptions:
      minimizeChanges:
        description: Minimize Changes settings.
        suboptions:
          enabled:
            description: Toggle for enabling or disabling Busy Hour in a network.
            type: bool
        type: dict
      schedule:
        description: Busy hour mode settings.
        suboptions:
          manual:
            description: Manual Busy Hour settings.
            suboptions:
              end:
                description: The hour that Manual Busy Hour ends each day, in the
                  network time zone.
                type: str
              start:
                description: The hour that Manual Busy Hour starts each day, in the
                  network time zone.
                type: str
            type: dict
          mode:
            description: The Busy Hour mode applied to the network when minimizeChanges
              is enabled. Must be one of 'automatic' or 'manual'. Automatic busy hour
              is only available on firmware versions >= MR 27.0.
            type: str
        type: dict
    type: dict
  channel:
    description: Channel settings.
    suboptions:
      avoidance:
        description: Avoidance settings.
        suboptions:
          enabled:
            description: Toggle for enabling or disabling channel avoidance in a network.
            type: bool
        type: dict
    type: dict
  fra:
    description: FRA settings.
    suboptions:
      enabled:
        description: Toggle to activate or deactivate FRA in a network, contingent
          on AI-RRM being enabled.
        type: bool
    type: dict
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless updateNetworkWirelessRadioRrm
    description: Complete reference of the updateNetworkWirelessRadioRrm API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-wireless-radio-rrm
notes:
  - SDK Method used are
    wireless.Wireless.update_network_wireless_radio_rrm,
  - Paths used are
    put /networks/{networkId}/wireless/radio/rrm,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_wireless_radio_rrm:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    ai:
      enabled: true
    busyHour:
      minimizeChanges:
        enabled: true
      schedule:
        manual:
          end: '15:00'
          start: '10:00'
        mode: automatic
    channel:
      avoidance:
        enabled: true
    fra:
      enabled: false
    networkId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "networkId": "string",
      "name": "string",
      "timeZone": "string",
      "busyHour": {
        "schedule": {
          "mode": "string",
          "automatic": {
            "start": "string",
            "end": "string"
          },
          "manual": {
            "start": "string",
            "end": "string"
          }
        },
        "minimizeChanges": {
          "enabled": true
        }
      },
      "channel": {
        "avoidance": {
          "enabled": true
        }
      },
      "fra": {
        "enabled": true
      },
      "ai": {
        "enabled": true,
        "lastEnabledAt": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_reboot
short_description: Reboot a windows machine
description:
- Unconditionally reboot a Windows machine, wait for it to go down, come back up, and respond to commands.
- For non-Windows targets, use the M(ansible.builtin.reboot) module instead.
options:
  pre_reboot_delay:
    description:
    - Seconds to wait before reboot. Passed as a parameter to the reboot command.
    - The minimum version is C(2) seconds and cannot be set lower.
    type: float
    default: 2
    aliases: [ pre_reboot_delay_sec ]
  post_reboot_delay:
    description:
    - Seconds to wait after the reboot command was successful before attempting to validate the system rebooted successfully.
    - This is useful if you want wait for something to settle despite your connection already working.
    type: float
    default: 0
    aliases: [ post_reboot_delay_sec ]
  reboot_timeout:
    description:
    - Maximum seconds to wait for machine to re-appear on the network and respond to a test command.
    - This timeout is evaluated separately for both reboot verification and test command success so maximum clock time is actually twice this value.
    type: float
    default: 600
    aliases: [ reboot_timeout_sec ]
  connect_timeout:
    description:
    - Maximum seconds to wait for a single successful TCP connection to the WinRM endpoint before trying again.
    type: float
    default: 5
    aliases: [ connect_timeout_sec ]
  test_command:
    description:
    - Command to expect success for to determine the machine is ready for management.
    - By default this test command is a custom one to detect when the Windows Logon screen is up and ready to accept
      credentials. Using a custom command will replace this behaviour and just run the command specified.
    type: str
  msg:
    description:
    - Message to display to users.
    type: str
    default: Reboot initiated by Ansible
  boot_time_command:
    description:
      - Command to run that returns a unique string indicating the last time the system was booted.
      - Setting this to a command that has different output each time it is run will cause the task to fail.
    type: str
    default: '(Get-CimInstance -ClassName Win32_OperatingSystem -Property LastBootUpTime).LastBootUpTime.ToFileTime()'
notes:
- If a shutdown was already scheduled on the system, M(ansible.windows.win_reboot) will abort the scheduled shutdown and enforce its own shutdown.
- Beware that when M(ansible.windows.win_reboot) returns, the Windows system may not have settled yet and some base services could be in limbo.
  This can result in unexpected behavior. Check the examples for ways to mitigate this. This has been slightly mitigated
  in the C(1.6.0) release of C(ansible.windows) but it is not guranteed to always wait until the logon prompt is shown.
- The connection user must have the C(SeRemoteShutdownPrivilege) privilege enabled, see
  U(https://docs.microsoft.com/en-us/windows/security/threat-protection/security-policy-settings/force-shutdown-from-a-remote-system)
  for more information.
- This module is equivalent to using the /f forced option for reboot on a windows host
seealso:
- module: ansible.builtin.reboot
author:
- Matt Davis (@nitzmahone)
'''

EXAMPLES = r'''
- name: Reboot the machine with all defaults
  ansible.windows.win_reboot:

- name: Reboot a slow machine that might have lots of updates to apply
  ansible.windows.win_reboot:
    reboot_timeout: 3600

# Install a Windows feature and reboot if necessary
- name: Install IIS Web-Server
  ansible.windows.win_feature:
    name: Web-Server
  register: iis_install

- name: Reboot when Web-Server feature requires it
  ansible.windows.win_reboot:
  when: iis_install.reboot_required

- name: Reboot with custom connect timeout for unreliable networks
  ansible.windows.win_reboot:
    msg: "Scheduled reboot for patching"
    connect_timeout: 10.0
    reboot_timeout: 900
    post_reboot_delay: 60

# One way to ensure the system is reliable, is to set WinRM to a delayed startup
- name: Ensure WinRM starts when the system has settled and is ready to work reliably
  ansible.windows.win_service:
    name: WinRM
    start_mode: delayed

# Additionally, you can add a delay before running the next task
- name: Reboot a machine that takes time to settle after being booted
  ansible.windows.win_reboot:
    post_reboot_delay: 120

# Or you can make win_reboot validate exactly what you need to work before running the next task
- name: Validate that the netlogon service has started, before running the next task
  ansible.windows.win_reboot:
    test_command: 'exit (Get-Service -Name Netlogon).Status -ne "Running"'
'''

RETURN = r'''
rebooted:
  description: True if the machine was rebooted.
  returned: always
  type: bool
  sample: true
elapsed:
  description: The number of seconds that elapsed waiting for the system to be rebooted.
  returned: always
  type: float
  sample: 23.2
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2021, Shreyas Srish (@shrsr) <ssrish@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_schema_template_bd_dhcp_policy
short_description: Manage BD DHCP Policy in schema templates
description:
- Manage BD DHCP policies in schema templates on Cisco ACI Multi-Site.
author:
- Shreyas Srish (@shrsr)
options:
  schema:
    description:
    - The name of the schema.
    type: str
    required: true
  template:
    description:
    - The name of the template to change.
    type: str
    required: true
  bd:
    description:
    - The name of the BD to manage.
    type: str
    required: true
  dhcp_policy:
    description:
    - The DHCP Policy
    type: str
    aliases: [ name ]
  version:
    description:
    - The version of DHCP Relay Policy.
    type: int
  dhcp_option_policy:
    description:
    - The DHCP Option Policy.
    type: dict
    suboptions:
      name:
        description:
        - The name of the DHCP Option Policy.
        type: str
        required: true
      version:
        description:
        - The version of the DHCP Option Policy.
        type: int
        required: true
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
notes:
- This module can only be used on versions of MSO that are 3.1.1h or greater.
- This module can only be used on versions of NDO that are 3.7.2i or lower.
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Add a new DHCP policy to a BD
  cisco.mso.mso_schema_template_bd_dhcp_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    bd: BD 1
    name: ansible_test
    version: 1
    dhcp_option_policy:
      name: ansible_test_option
      version: 1
    state: present

- name: Remove a DHCP policy from a BD
  cisco.mso.mso_schema_template_bd_dhcp_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    bd: BD 1
    name: ansible_test
    version: 1
    state: absent

- name: Query a specific BD DHCP Policy
  cisco.mso.mso_schema_template_bd_dhcp_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    bd: BD 1
    name: ansible_test
    state: query
  register: query_result

- name: Query all BD DHCP Policies
  cisco.mso.mso_schema_template_bd_dhcp_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    bd: BD 1
    state: query
  register: query_result
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec, mso_dhcp_option_spec


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type="str", required=True),
        template=dict(type="str", required=True),
        bd=dict(type="str", required=True),
        dhcp_policy=dict(type="str", aliases=["name"]),
        version=dict(type="int"),
        dhcp_option_policy=dict(type="dict", options=mso_dhcp_option_spec()),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["dhcp_policy"]],
            ["state", "present", ["dhcp_policy", "version"]],
        ],
    )

    schema = module.params.get("schema")
    template = module.params.get("template").replace(" ", "")
    bd = module.params.get("bd")
    dhcp_policy = module.params.get("dhcp_policy")
    dhcp_option_policy = module.params.get("dhcp_option_policy")
    version = module.params.get("version")
    state = module.params.get("state")

    mso = MSOModule(module)

    # Get schema
    schema_id, schema_path, schema_obj = mso.query_schema(schema)

    # Get template
    templates = [t.get("name") for t in schema_obj.get("templates")]
    if template not in templates:
        mso.fail_json(msg="Provided template '{0}' does not exist. Existing templates: {1}".format(template, ", ".join(templates)))
    template_idx = templates.index(template)

    # Get BD
    bds = [b.get("name") for b in schema_obj.get("templates")[template_idx]["bds"]]
    if bd not in bds:
        mso.fail_json(msg="Provided BD '{0}' does not exist. Existing BDs: {1}".format(bd, ", ".join(bds)))
    bd_idx = bds.index(bd)

    # Check if DHCP policy already exists
    if dhcp_policy:
        check_policy = mso.get_obj("policies/dhcp/relay", name=dhcp_policy, key="DhcpRelayPolicies")
        if check_policy:
            pass
        else:
            mso.fail_json(msg="DHCP policy '{dhcp_policy}' does not exist".format(dhcp_policy=dhcp_policy))

    # Check if DHCP option policy already exists
    if dhcp_option_policy:
        check_option_policy = mso.get_obj("policies/dhcp/option", name=dhcp_option_policy.get("name"), key="DhcpRelayPolicies")
        if check_option_policy:
            pass
        else:
            mso.fail_json(msg="DHCP option policy '{dhcp_option_policy}' does not exist".format(dhcp_option_policy=dhcp_option_policy.get("name")))

    # Get DHCP policies
    dhcp_policies = [s.get("name") for s in schema_obj.get("templates")[template_idx]["bds"][bd_idx]["dhcpLabels"]]
    if dhcp_policy in dhcp_policies:
        dhcp_idx = dhcp_policies.index(dhcp_policy)
        # FIXME: Changes based on index are DANGEROUS
        dhcp_policy_path = "/templates/{0}/bds/{1}/dhcpLabels/{2}".format(template, bd, dhcp_idx)
        mso.existing = schema_obj.get("templates")[template_idx]["bds"][bd_idx]["dhcpLabels"][dhcp_idx]

    if state == "query":
        if dhcp_policy is None:
            mso.existing = schema_obj.get("templates")[template_idx]["bds"][bd_idx]["dhcpLabels"]
        elif not mso.existing:
            mso.fail_json(msg="DHCP policy not associated with the bd")
        mso.exit_json()

    dhcp_policy_paths = "/templates/{0}/bds/{1}/dhcpLabels".format(template, bd)
    ops = []

    mso.previous = mso.existing
    if state == "absent":
        if mso.existing:
            mso.sent = mso.existing = {}
            ops.append(dict(op="remove", path=dhcp_policy_path))

    elif state == "present":
        payload = dict(
            name=dhcp_policy,
            version=version,
            dhcpOptionLabel=dhcp_option_policy,
        )

        mso.sanitize(payload, collate=True)

        if mso.existing:
            ops.append(dict(op="replace", path=dhcp_policy_path, value=mso.sent))
        else:
            ops.append(dict(op="add", path=dhcp_policy_paths + "/-", value=mso.sent))

        mso.existing = mso.proposed

    if not module.check_mode:
        mso.request(schema_path, method="PATCH", data=ops)

    mso.exit_json()


if __name__ == "__main__":
    main()
